/* lpcanalysis.s */
/**********************************************************************

                                lpcanalysis()

***********************************************************************

        inputs:         (One channel)

        outputs:        (optional feed-through of input channels)

        parameters:     int npts, the number of points to plot
                        int skip, number of points to skip before first plot
                        file title,  the title of the plot
                        dB flag (0=linear 1=dB)
                        window flag (0=Rect 1= Hamming )

*************************************************************************

This routine produces the lpcanalysis 
of the input buffer.

The first parameter represents the number of points plotted from the channel.
Default is set to (int 128).
The second parameter is the number of points to skip before the first plot set
is presented.  This enables skipping of any transient period.
Default is set to (int 0).
The third parameter represents the title for the plot.
Default is set to "PLOT".
The fourth parameter is the dB flag (0 for linear, 1 for dB );
The fifth parameter is the window flag ( 0 for Rectangular 1 for hammming)


Programmer: 	Sasan Ardalan
Date: 		Dec. 4, 1993	
*/

#include "iiplot.h"


#define FLOAT_BUFFER 0
#define INTEGER_BUFFER 1

#define RECTANGULAR_WINDOW 0
#define HAMMING_WINDOW 1

#define PI 3.1415926

#define BLOCK_SIZE 512

parameters
	param_def="Number of points to skip";
	int skip=0 ;	/* points to skip before first plot */
	param_def="Method: 0=AUTO, 1=COVAR";
	int method=0 ;	
	param_def="LPC Order";
	int m=8 ;	
	param_def="Window size (samples)";
	int windowLength=128 ;	
	param_def="Zero padding (samples)";
	int zeropad=0 ;	
	param_def = "Sampling Rate (KHz)";
	float	samplingRate=8;
	param_def = "Linear = 0, dB = 1";
	int	dBFlag = 1;
	param_def = "Plot Expanded Spectrum:1=True 0=False";
	int	expandFlag = 1;
	param_def = "Points per Spectrum";
	int	npts = 100;
	param_def = "Plot Residual: 1=True 0=False";
	int	residualFlag = 0;
	param_def = "Plot Poles: 1=True 0=False";
	int	poleZeroFlag = 0;
	param_def = "Plot Formants: 1=True 0=False";
	int	formantFlag = 0;
	param_def = "Plot Zero Crossing: 1=True 0=False";
	int	zeroCrossingFlag = 0;
	param_def = "Plot Stacked Reflection Coeff: 1=True 0=False";
	int	reflectionFlag = 0;
	param_def = "Surface Plot: 1=True 0=False";
	int	surfaceFlag = 0;
	param_def = "Image: 1=True 0=False";
	int	imageFlag = 0;
	param_def="Plot title";
	file spectTitle = "LPC_Analysis";
	param_def = "Time Domain On/Off (1/0)";
	int	timeFlag=1;
	param_def ="Control: 1=On, 0=Off";
	int	control = 1;
        param_def = "Buffer type:0= Float,1=Integer";
        int     bufferType=0;
end

states
	int numberInputBuffers;	/* number of input buffers connected */
	int numberOutputBuffers;	/* number of output buffers connected */
    	float* xpts;	/* array for input x data */
    	float* xTime_P;	/* array for input x data */
    	float* ypts;	/* array for input y data */
    	float* spect_P;	/* array for lpcanalysis */
    	int count = 0;
	int totalCount = 0;
	int             blockOff;
	int		bufi;

	float	dx;
end

declarations
        iip_Curve_Pt     curveTime_P;
        iip_Curve_Pt     curve_P;
	int samples;
	int numberPoints;
    	int i,j,ii;
	float tmp;
	complex val;
	float wind;
	char title1[80];
	char fname[80];
        char curveSubTitle[80];
	float* mag_P;
	float* phase_P;
	FILE *time_F;
	FILE *freq_F;
end

initialization_code
	                         

/* store as state the number of input/output buffers */
if((numberInputBuffers = no_input_buffers()) <= 0) {
	fprintf(stderr,"lpcanalysis: no inputs connected\n");
	return(1);
}
if(numberInputBuffers > 1) {
	fprintf(stderr,"lpcanalysis: only one  input allowed \n");
	return(2);
}
if((numberOutputBuffers = no_output_buffers()) > numberInputBuffers) {
	fprintf(stderr,"lpcanalysis: too many outputs connected\n");
	return(3);
}

if(samplingRate)
	dx=1.0/samplingRate;
else
	dx=1.0;



if (control) {
	/*                                                      
	 * allocate arrays                                      
	 */                                                     
	xpts = (float* )calloc(BLOCK_SIZE,sizeof(float));           
	ypts = (float* )calloc(BLOCK_SIZE,sizeof(float));           
	if(xpts==NULL  || ypts ==NULL) {
		fprintf(stderr,"Spectrogram could not allocate space\n");
		return(4);
	}
}
count = 0;
totalCount = 0;

switch(bufferType) {
	case FLOAT_BUFFER: 
		set_cellsize_in(0,sizeof(float));
		if(numberOutputBuffers == 1)
			set_cellsize_out(0,sizeof(float));
		break;
	case INTEGER_BUFFER: 
		set_cellsize_in(0,sizeof(int));
		if(numberOutputBuffers == 1)
			set_cellsize_out(0,sizeof(int));
		break;
	default: 
		fprintf(stderr,"Bad buffer type specified in lpcanalysis \n");
		return(5);
		break;
}


end


main_code

for(samples = min_avail(); samples > 0; --samples) {

  for(i=0; i<numberInputBuffers; ++i) {
   		it_in(i);
		if(numberOutputBuffers > i) {
			if(it_out(i)) {
				KrnOverflow("lpcanalysis",i);
				return(99);
			}

			switch(bufferType) {
				case INTEGER_BUFFER:
	 				outi(i,0) = ini(i,0);
					break;
				case FLOAT_BUFFER:
	 				outf(i,0) = inf(i,0);
					break;
			}

		}
  }
  if(++totalCount > skip && control ) {
		count=blockOff + bufi;
		bufi++;
		if (bufi == BLOCK_SIZE) {
			blockOff += BLOCK_SIZE;
			xpts = (float *)realloc((char *) xpts, 
				sizeof(float) * (blockOff + BLOCK_SIZE));
			ypts = (float *)realloc((char *) ypts, 
				sizeof(float) * (blockOff + BLOCK_SIZE));
			bufi=0;
		}
		if(xpts == NULL || ypts==NULL) {
			fprintf(stderr,"lpcanalysis: could not allocate space\n");
			return(6);
		}

		switch(bufferType) {
			case INTEGER_BUFFER:
          	     		ypts[count] = ini(0,0);
				break;
			case FLOAT_BUFFER:
          	     		ypts[count] = inf(0,0);
				break;
		}
		xpts[count]=count*dx;

		
  }

} 

return(0);

end

wrapup_code
         
if(control == 0) return(0);

if((totalCount - skip) > 0 ) {
   /*
    * callup the plot routine 
    */
   if(iip_hdr_P ) {
	/*
	 * if in graphic mode callup iip
	 */
	 xpts = (float *) realloc((char *) xpts, sizeof(float) *
				 (blockOff + bufi));
	 ypts = (float *) realloc((char *) ypts, sizeof(float) *
				 (blockOff + bufi));
	if(xpts == NULL || ypts==NULL) {
		fprintf(stderr,"lpcanalysis: could not allocate space\n");
		return(6);
	}
	curveTime_P = IIP_NewInitedCurveRec();
	curveTime_P->qpCurveFlag = TRUE;
	curveTime_P->x_P = xpts;
	curveTime_P->y_P = ypts;
	curveTime_P->points = count;
	curveTime_P->note_A = NULL;
        curveTime_P->funcType = PLT_GROUP_TIME_DOMAIN;


	sprintf(title1,"%s Time Domain %s",spectTitle,sname(0));
	IIP_SetTitles(curveTime_P,title1,NULL);
	if(timeFlag)
		IIP_AppendCurve(curveTime_P);
	IIP_LPCAnalysis(curveTime_P,method+1,m,windowLength,npts,0,
		samplingRate, dBFlag,expandFlag,surfaceFlag,
			imageFlag,reflectionFlag,residualFlag,
			poleZeroFlag,formantFlag,zeroCrossingFlag,
			0,NULL, 0,NULL);

   }
}
end
   
                    
