/* spectrogram.s */
/**********************************************************************

                                spectrogram()

***********************************************************************

        inputs:         (One channel)

        outputs:        (optional feed-through of input channels)


*************************************************************************

This routine produces the spectrogram 
of the input buffer.



Programmer: 	Sasan Ardalan
Date: 		Dec. 4, 1993	
*/

#include "iiplot.h"


#define FLOAT_BUFFER 0
#define INTEGER_BUFFER 1

#define RECTANGULAR_WINDOW 0
#define HAMMING_WINDOW 1

#define PI 3.1415926

#define BLOCK_SIZE 512

parameters
	param_def="Number of points";
	int npts=128 ;	/* number of buffer points in each plot */
	param_def="Number of points to skip";
	int skip=0 ;	/* points to skip before first plot */
	param_def="Window size (samples)";
	int windowLength=128 ;	
	param_def="Overlap (samples)";
	int overlap=0 ;	
	param_def="Zero padding (samples)";
	int zeropad=0 ;	
	param_def = "Sampling Rate (KHz)";
	float	samplingRate=8;
	param_def = "Window:0= Rec.,1=Hamming";
	int	windowType = 1;
	param_def = "Linear = 0, dB = 1";
	int	dBFlag = 1;
	param_def = "Surface Plot: 1=True 0=False";
	int	surfaceFlag = 0;
	param_def = "Image: 1=True 0=False";
	int	imageFlag = 1;
	param_def="Plot title";
	file spectTitle = "Spectrogram";
	param_def = "Time Domain On/Off (1/0)";
	int	timeFlag=1;
	param_def ="Control: 1=On, 0=Off";
	int	control = 1;
        param_def = "Buffer type:0= Float,1=Integer";
        int     bufferType=0;
end

states
	int numberInputBuffers;	/* number of input buffers connected */
	int numberOutputBuffers;	/* number of output buffers connected */
    	float* xpts;	/* array for input x data */
    	float* xTime_P;	/* array for input x data */
    	float* ypts;	/* array for input y data */
    	float* spect_P;	/* array for spectrogram */
    	int count = 0;
	int totalCount = 0;
	int             blockOff=0;
	int		bufi=0;

	float	dx;
end

declarations
        iip_Curve_Pt     curveTime_P;
        iip_Curve_Pt     curve_P;
	int samples;
	int numberPoints;
    	int i,j,ii;
	float tmp;
	complex val;
	float wind;
	char title1[80];
	char fname[80];
        char curveSubTitle[80];
	float* mag_P;
	float* phase_P;
	FILE *time_F;
	FILE *freq_F;
end

initialization_code
	                         

/* store as state the number of input/output buffers */
if((numberInputBuffers = no_input_buffers()) <= 0) {
	fprintf(stderr,"spectrogram: no inputs connected\n");
	return(1);
}
if(numberInputBuffers > 1) {
	fprintf(stderr,"spectrogram: only one  input allowed \n");
	return(2);
}
if((numberOutputBuffers = no_output_buffers()) > numberInputBuffers) {
	fprintf(stderr,"spectrogram: too many outputs connected\n");
	return(3);
}

if(samplingRate)
	dx=1.0/samplingRate;
else
	dx=1.0;



if (control) {
	/*                                                      
	 * allocate arrays                                      
	 */                                                     
	xpts = (float* )calloc(BLOCK_SIZE,sizeof(float));           
	ypts = (float* )calloc(BLOCK_SIZE,sizeof(float));           
	if(xpts==NULL  || ypts ==NULL) {
		fprintf(stderr,"Spectrogram could not allocate space\n");
		return(4);
	}
}
count = 0;
totalCount = 0;

switch(bufferType) {
	case FLOAT_BUFFER: 
		set_cellsize_in(0,sizeof(float));
		if(numberOutputBuffers == 1)
			set_cellsize_out(0,sizeof(float));
		break;
	case INTEGER_BUFFER: 
		set_cellsize_in(0,sizeof(int));
		if(numberOutputBuffers == 1)
			set_cellsize_out(0,sizeof(int));
		break;
	default: 
		fprintf(stderr,"Bad buffer type specified in spectrogram \n");
		return(5);
		break;
}


end


main_code

for(samples = min_avail(); samples > 0; --samples) {

  for(i=0; i<numberInputBuffers; ++i) {
   		it_in(i);
		if(numberOutputBuffers > i) {
			if(it_out(i)) {
				KrnOverflow("spectrogram",i);
				return(99);
			}

			switch(bufferType) {
				case INTEGER_BUFFER:
	 				outi(i,0) = ini(i,0);
					break;
				case FLOAT_BUFFER:
	 				outf(i,0) = inf(i,0);
					break;
			}

		}
  }
  if(++totalCount > skip && control ) {
		count=blockOff + bufi;
		bufi++;
		if (bufi == BLOCK_SIZE) {
			blockOff += BLOCK_SIZE;
			xpts = (float *)realloc((char *) xpts, 
				sizeof(float) * (blockOff + BLOCK_SIZE));
			ypts = (float *)realloc((char *) ypts, 
				sizeof(float) * (blockOff + BLOCK_SIZE));
			bufi=0;
		}
		if(xpts == NULL || ypts==NULL) {
			fprintf(stderr,"spectrogram: could not allocate space\n");
			return(6);
		}

		switch(bufferType) {
			case INTEGER_BUFFER:
          	     		ypts[count] = ini(0,0);
				break;
			case FLOAT_BUFFER:
          	     		ypts[count] = inf(0,0);
				break;
		}
		xpts[count]=count*dx;

		
  }

} 

return(0);

end

wrapup_code
         
if(control == 0) return(0);

if((totalCount - skip) > 0 ) {
   /*
    * callup the plot routine 
    */
   if(iip_hdr_P ) {
	/*
	 * if in graphic mode callup iip
	 */
	 xpts = (float *) realloc((char *) xpts, sizeof(float) *
				 (blockOff + bufi));
	 ypts = (float *) realloc((char *) ypts, sizeof(float) *
				 (blockOff + bufi));
	if(xpts == NULL || ypts==NULL) {
		fprintf(stderr,"spectrogram: could not allocate space\n");
		return(6);
	}
	curveTime_P = IIP_NewInitedCurveRec();
	curveTime_P->qpCurveFlag = TRUE;
	curveTime_P->x_P = xpts;
	curveTime_P->y_P = ypts;
	curveTime_P->points = count;
	curveTime_P->note_A = NULL;
        curveTime_P->funcType = PLT_GROUP_TIME_DOMAIN;


	sprintf(title1,"%s Time Domain %s",spectTitle,sname(0));
	IIP_SetTitles(curveTime_P,title1,NULL);
	if(timeFlag)
		IIP_AppendCurve(curveTime_P);
		curve_P=IIP_Spectrogram(curveTime_P,windowLength,overlap,
				zeropad,
				samplingRate, windowType,surfaceFlag,
				imageFlag,dBFlag,0,NULL);
	if(curve_P)
		strcpy(curve_P->title,spectTitle);

   }
}
end
   
                    
