
# topology file:  deltasigma.t

#--------------------------------------------------- 
# Title: Delta Sigma Modulation (2nd Order)  
# Author: Sasan H. Ardalan  
# Date: July 4, 1992  
# Description: Demonstrates 2nd order delta sigma modulation with decimation   
#--------------------------------------------------- 

inform title Delta Sigma Modulation (2nd Order) 
inform author Sasan H. Ardalan 
inform date July 4, 1992 
inform descrip Demonstrates 2nd order delta sigma modulation with decimation  

arg 0 int 65000 "Number of samples high rate"
arg 1 int 256 "Number of decimated samples"
arg 2 int 128 "1/DecimationRatio"

param arg 0    num_of_samples   "Number of samples high rate"
param float 1    magnitude   "Enter Magnitude"
param float 2.048e+06    fs   "Enter Sampling Rate"
param float 2000    freq   "Enter Frequency"
param float 0    phase   "Enter Phase"
param float 1    pace_rate   "pace rate to determine how many samples to output"
param int 128    samples_first_time   "number of samples on the first call if paced"
block sine0 sine

param float 0.5    g1   "The gain of the first integrator"
param float 1    g2   "The gain of the second integrator"
param float 1    delta   "Binary Quantizer Level"
block ds20 ds2

param arg 0    npts   "Number of samples high rate"
param int 0    skip   "Number of points to skip"
param file P(k)    title   "Plot title"
param int 0    dbFlag   "Linear = 0, dB = 1"
param int 0    windFlag   "Window:0= Rec.,1=Hamm,2=Hann,3=Blackman"
param int 1    plotStyleParam   "Plot Style: 1=Line,2=Points,5=Bar Chart"
param int 1    timeFlag   "Time Domain On/Off (1/0)"
param float 2.048    sampFreq   "Sampling Rate (Bin Number if zero,Normalized if Negative"
param int 1    control   "Control: 1=On, 0=Off"
param int 0    bufferType   "Buffer type:0= Float,1= Complex, 2=Integer"
param int 0    mode   "0=Static,1=Dynamic"
param string "Frequency"    xLabelSpectrum   "Spectrum X Axis  Label "
param string "Time"    xLabelTime   "Time X Axis  Label "
block xspectrum0 spectrum

param int 2    iw   "Weighting: 1-Uniform 2-Triangular 3-Parabolic"
param int 128    nf   "Number of taps for FIR decimation filter"
param int 64    idec   "Decimation factor"
block decimate0 decimate

param arg 1    npts   "Number of decimated samples"
param int 0    skip   "Number of points to skip"
param file Decimated    title   "Plot title"
param int 0    dbFlag   "Linear = 0, dB = 1"
param int 0    windFlag   "Window:0= Rec.,1=Hamm,2=Hann,3=Blackman"
param int 1    plotStyleParam   "Plot Style: 1=Line,2=Points,5=Bar Chart"
param int 1    timeFlag   "Time Domain On/Off (1/0)"
param float 0    sampFreq   "Sampling Rate (Bin Number if zero,Normalized if Negative"
param int 1    control   "Control: 1=On, 0=Off"
param int 0    bufferType   "Buffer type:0= Float,1= Complex, 2=Integer"
param int 0    mode   "0=Static,1=Dynamic"
param string "Frequency"    xLabelSpectrum   "Spectrum X Axis  Label "
param string "Time"    xLabelTime   "Time X Axis  Label "
block spectrum1 spectrum

param int 3    desType   "1=Butterworth,2=Chebyshev,3=Elliptic"
param int 1    filterType   "1=LowPass,2=HighPass,3=BandPass,4=BandStop"
param float 32000    fs   "Sampling Frequency, Hz"
param float 0.1    pbdb   "Enter Passband Ripple in dB's"
param float 60    sbdb   "Enter Stopband Attenuation in dB's"
param float 3400    fpb   "Passband Freq. Hz/LowPass/HighPass Only"
param float 4400    fsb   "Stopband Freq. Hz/LowPass/HighPass Only"
param float 220    fpl   "Lower Passband Freq. Hz/BandPass/BandStop Only"
param float 3400    fpu   "Upper Passband Freq. Hz/BandPass/BandStop Only"
param float 10    fsl   "Lower Stopband Freq. Hz/BandPass/BandStop Only"
param float 4400    fsu   "Upper Stopband Freq. Hz/BandPass/BandStop Only"
param file tmp    filterName   "Filter Name"
block iirfil0 iirfil

param int 256    npts   "Number of points to collect"
param int 128    skip   "Number of points to skip"
param file sdr.dat    sdrRes   "File to store SDR results"
param int 1    windFlag   "Window: 0=Rect., 1=Hamming"
block sdr0 sdr

connect sine0 0 ds20 0  	{sine0:NULL:NULL,ds20:in:float}
connect ds20 0 xspectrum0 0  	p(k)
connect xspectrum0 0 decimate0 0  	{xspectrum0:NULL:NULL,decimate0:xx:float}
connect decimate0 0 spectrum1 0  	
connect spectrum1 0 iirfil0 0  	{spectrum1:NULL:NULL,iirfil0:x:float}
connect iirfil0 0 sdr0 0  	

