
# topology file:  twt_measurement.t

#--------------------------------------------------- 
# Title: TWT using Saleh Model 
# Author: Sasan H. Ardalan  
# Date: June 25, 2007  
# Description: Implements TWT using Saleh model based on Fig.4 in IEEE Trans. Comm. , Nov. 1981, pages 1715-1720 
#--------------------------------------------------- 

inform title TWT using Saleh Model
inform author Sasan H. Ardalan 
inform date June 25, 2007 
inform descrip Implements TWT using Saleh model based on Fig.4 in IEEE Trans. Comm. , Nov. 1981, pages 1715-1720

arg 0 int 4096 "Number of Samples"

param arg 0    num_of_samples   "Number of Samples"
param float 1    magnitude   "Enter Magnitude"
param float 100    fs   "Enter Sampling Rate"
param float 10    freq   "Enter Frequency"
param float 0    phase   "Enter Phase"
param float 1    pace_rate   "pace rate to determine how many samples to output"
param int 128    samples_first_time   "number of samples on the first call if paced"
block sine0 sine

param arg 0    num_of_samples   "Number of Samples"
param float 2    magnitude   "Enter Magnitude"
param float 100    fs   "Enter Sampling Rate"
param float 0.1    freq   "Enter Frequency"
param float 0    phase   "Enter Phase"
param float 1    pace_rate   "pace rate to determine how many samples to output"
param int 128    samples_first_time   "number of samples on the first call if paced"
block sine1 sine

block node0 node

block node1 node

param float 1.6623    alphaA   " alphaA "
param float 0.0552    betaA   " betaA "
param float 0.1533    alphaPhi   " alphaPhi "
param float 0.3456    betaPhi   " betaPhi "
block nonlinsaleh0 nonlinsaleh

param int 0    skip   "Points to skip"
param file stat.dat    stat_file   "File to store results"
block nlstat stats

block node2 node

param int 128    npts   "Number of points ( dynamic window size )"
param int 0    skip   "Number of points to skip"
param file Spectrum    title   "Plot title"
param int 1    dbFlag   "Linear = 0, dB = 1"
param int 0    windFlag   "Window:0= Rec.,1=Hamm,2=Hann,3=Blackman"
param int 1    plotStyleParam   "Plot Style: 1=Line,2=Points,5=Bar Chart"
param int 1    timeFlag   "Time Domain On/Off (1/0)"
param float 100    sampFreq   "Sampling Rate (Bin Number if zero,Normalized if Negative"
param int 1    control   "Control: 1=On, 0=Off"
param int 0    bufferType   "Buffer type:0= Float,1= Complex, 2=Integer"
param int 0    mode   "0=Static,1=Dynamic"
param string "Frequency"    xLabelSpectrum   "Spectrum X Axis  Label "
param string "Time"    xLabelTime   "Time X Axis  Label "
block nlspectrum spectrum

block mixer0 mixer

block mixer1 mixer

param int 3    desType   "1=Butterworth,2=Chebyshev,3=Elliptic"
param int 1    filterType   "1=LowPass,2=HighPass,3=BandPass,4=BandStop"
param float 32000    fs   "Sampling Frequency, Hz"
param float 0.1    pbdb   "Enter Passband Ripple in dB's"
param float 50    sbdb   "Enter Stopband Attenuation in dB's"
param float 3400    fpb   "Passband Freq. Hz/LowPass/HighPass Only"
param float 4400    fsb   "Stopband Freq. Hz/LowPass/HighPass Only"
param float 220    fpl   "Lower Passband Freq. Hz/BandPass/BandStop Only"
param float 3400    fpu   "Upper Passband Freq. Hz/BandPass/BandStop Only"
param float 10    fsl   "Lower Stopband Freq. Hz/BandPass/BandStop Only"
param float 4400    fsu   "Upper Stopband Freq. Hz/BandPass/BandStop Only"
param file tmp    filterName   "Filter Name"
block iirfil0 iirfil

param int 3    desType   "1=Butterworth,2=Chebyshev,3=Elliptic"
param int 1    filterType   "1=LowPass,2=HighPass,3=BandPass,4=BandStop"
param float 32000    fs   "Sampling Frequency, Hz"
param float 0.1    pbdb   "Enter Passband Ripple in dB's"
param float 50    sbdb   "Enter Stopband Attenuation in dB's"
param float 3400    fpb   "Passband Freq. Hz/LowPass/HighPass Only"
param float 4400    fsb   "Stopband Freq. Hz/LowPass/HighPass Only"
param float 220    fpl   "Lower Passband Freq. Hz/BandPass/BandStop Only"
param float 3400    fpu   "Upper Passband Freq. Hz/BandPass/BandStop Only"
param float 10    fsl   "Lower Stopband Freq. Hz/BandPass/BandStop Only"
param float 4400    fsu   "Upper Stopband Freq. Hz/BandPass/BandStop Only"
param file tmp    filterName   "Filter Name"
block iirfil1 iirfil

block ang0 ang

param int 0    skip   "Points to skip"
param file stat.dat    stat_file   "File to store results"
block angstats stats

param int 128    npts   "Number of points in each plot (dynamic window size)"
param int 0    skip   "Points to skip before first plot"
param file plot    title   "Plot title"
param file Samples    x_axis   "X Axis label"
param file Y    y_axis   "Y-Axis label"
param int 1    plotStyleParam   "Plot Style: 1=Line,2=Points,5=Bar Chart"
param int 1    control   "Control: 1=On, 0=Off"
param int 0    mode   "0=Static,1=Dynamic"
param int 1    samplingRate   "Sampling Rate"
block angplot plot

connect sine0 0 node0 0  	{sine0:NULL:NULL,node0:x:float}
connect sine0 1 node1 0  	{sine0:NULL:NULL,node1:x:float}
connect sine1 0 nonlinsaleh0 0  	{sine1:NULL:NULL,nonlinsaleh0: r : float }
connect node0 0 nonlinsaleh0 1  	{node0:NULL:NULL,nonlinsaleh0: scos : float }
connect node0 1 mixer0 1  	{node0:NULL:NULL,mixer0:y:float}
connect node1 0 nonlinsaleh0 2  	{node1:NULL:NULL,nonlinsaleh0: ssin : float }
connect node1 1 mixer1 1  	{node1:NULL:NULL,mixer1:y:float}
connect nonlinsaleh0 0 nlstat 0  	{nonlinsaleh0: y : float ,nlstat:x:float}
connect nlstat 0 node2 0  	{nlstat:NULL:NULL,node2:x:float}
connect node2 0 nlspectrum 0  	
connect node2 1 mixer0 0  	{node2:NULL:NULL,mixer0:x:float}
connect node2 2 mixer1 0  	{node2:NULL:NULL,mixer1:x:float}
connect mixer0 0 iirfil0 0  	{mixer0:z:float,iirfil0:x:float}
connect mixer1 0 iirfil1 0  	{mixer1:z:float,iirfil1:x:float}
connect iirfil0 0 ang0 0  	{iirfil0:y:float,ang0:x:float}
connect iirfil1 0 ang0 1  	{iirfil1:y:float,ang0:y:float}
connect ang0 0 angstats 0  	{ang0:z:float,angstats:x:float}
connect angstats 0 angplot 0  	

