
#
# (c) XCAD  Corporation, Portland, Oregon, All Rights Reserved
#

#
# TCL Script for measuring TWT AM-AM and AM-PM characteristics.
# The script stores the results of each run in a list.
# At the end of the iteration loop the results are tabulated.
#
#
# Reference:
# Saleh, A., "Frequency-Independent and Frequency-Dependent Nonlinear Models of TWT Amplifiers
# IEEE Transactions on Communications 
# Volume 29,  Issue 11,  Date: Nov 1981,  Pages:  1715 - 1720
#
#
# The script sets up the TWT (nonlinsaleh) block as in Figure 4 in the referenc.
#  A 10GHz carrier is used. The baseband is 100 MHz.
#
# The input voltage is in millivolst from 0 to 6 millivolts
#
# Results are stored in "twtresults.dat" in tab delimited format ready for import to 
# spreadsheet.
#
# Author: Sasan Ardalan
# June 25th, 2006
# Modified for TWT Measurement  June 27, 2007
#

new
capload twt_measurement.t

#turn the  probes off

to nlspectrum
parambyname control 0
to angplot
parambyname control 0

#
# setup nonlinear Saleh TWT model based on Figure 4 in reference
#
to nonlinsaleh0
parambyname alphaA 1.6623
parambyname betaA 0.0552
parambyname alphaPhi 0.1533
parambyname betaPhi 0.3456


set results [list {}]
# Sampling Rate 100 GHz
set fs 100 
to sine0
# 10GHz
parambyname freq 10   
parambyname magnitude 1.0
parambyname fs $fs

to sine1
# 100 MHz
parambyname freq 0.1   
parambyname magnitude 1.0
parambyname fs $fs


set deltaA 0.1

for { set i 1}  { $i < 61 } { incr i} {
   set amplitude [expr $i*$deltaA]


#
# set modulation amplitude in milli volts
#
   puts $amplitude
   to sine1
   parambyname magnitude $amplitude
   
   
    
   run
    
   puts $nlstat_max
   
   set outVolts  $nlstat_max
   set inVolts  $amplitude
   set angle    [expr $angstats_mean*180/3.1415926-180.0]
   lappend results [list $inVolts $outVolts $angle] 

}

puts -nonewline Input(millivolts)
puts -nonewline "\t"
puts   -nonewline Output(volts)
puts -nonewline "\t"
puts   angle(degrees)                          

foreach value $results {

   puts  -nonewline [lindex $value 0]
   puts  -nonewline "\t"
   puts   -nonewline [lindex $value  1]
   puts  -nonewline "\t"
   puts   [lindex $value  2]
}

set out [ open twtresults.dat w ]

foreach value $results {
	
   puts  -nonewline $out [lindex $value 0]
   puts  -nonewline $out "\t"
   puts  -nonewline $out [lindex $value  1]
   puts  -nonewline $out "\t"
   puts   $out [lindex $value  2]
  

}
close $out
