 
#ifdef LICENSE

/*  Capsim (r) Text Mode Kernel (TMK) Star Library (Blocks)
    Copyright (C) 1989-2002  XCAD Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.xcad.com
    XCAD Corporation
    Raleigh, North Carolina */

#endif
 
#ifdef SHORT_DESCRIPTION

This star simulates an Ethernet transceiver cable by introducing a fixed delay

#endif
 
#ifdef PROGRAMMERS

#endif






#include <stdio.h>
#include <math.h>

/* Note: these paths are installation dependent! */
#include <capsim.h>
#include <stars.h>



/*
 *           STATES STRUCTURE 
 */ 
typedef struct {
      int  __first;
     } state_t,*state_Pt;

/*
 *         STATE DEFINES 
 */ 
#define first (state_P->__first)

/*         
 *    INPUT BUFFER DEFINES 
 */ 
#define txIn(DELAY) (*((float  *)PIN(0,DELAY)))
#define rxIn(DELAY) (*((float  *)PIN(1,DELAY)))
#define cdIn(DELAY) (*((float  *)PIN(2,DELAY)))

/*         
 *    OUTPUT BUFFER  DEFINES 
 */ 
#define txOut(delay) *(float  *)POUT(0,delay)
#define rxOut(delay) *(float  *)POUT(1,delay)
#define cdOut(delay) *(float  *)POUT(2,delay)

/*         
 *    PARAMETER DEFINES 
 */ 
#define samplesDelay (param_P[0]->value.d)
/*-------------- BLOCK CODE ---------------*/


cable 

(run_state,block_P)

	int run_state;
	block_Pt block_P;
{
	param_Pt *param_P = block_P->param_AP;
	star_Pt star_P = block_P->star_P;

     
	state_Pt state_P = (state_Pt)star_P->state_P;
     
/*
 *              Declarations 
 */
 

	int	i;
	int	numberSamples;


switch (run_state) {

 
 


 
case PARAM_INIT: 
    {
      int indexModel88 = block_P->model_index;
     char   *pdef0 = "The delay of the cable in samples";
     char   *ptype0 = "int";
     char   *pval0 = "0";
     char   *pname0 = "samplesDelay";
KrnModelParam(indexModel88,0 ,pdef0,ptype0,pval0,pname0);

      }
break;
   


/*         
 *    OUTPUT BUFFER SYSTEM  INITS 
 */ 
 case OUTPUT_BUFFER_INIT: 
 {
 int indexOC = block_P->model_index;
     char   *ptypeOut0 = "float";
     char   *pnameOut0 = "txOut";
     char   *ptypeOut1 = "float";
     char   *pnameOut1 = "rxOut";
     char   *ptypeOut2 = "float";
     char   *pnameOut2 = "cdOut";
KrnModelConnectionOutput(indexOC,0 ,pnameOut0,ptypeOut0);
KrnModelConnectionOutput(indexOC,1 ,pnameOut1,ptypeOut1);
KrnModelConnectionOutput(indexOC,2 ,pnameOut2,ptypeOut2);
}
 break;

/*         
 *    INPUT BUFFER SYSTEM  INITS 
 */ 
 case INPUT_BUFFER_INIT:
 {
 int indexIC = block_P->model_index;
     char   *ptypeIn0 = "float";
     char   *pnameIn0 = "txIn";
     char   *ptypeIn1 = "float";
     char   *pnameIn1 = "rxIn";
     char   *ptypeIn2 = "float";
     char   *pnameIn2 = "cdIn";
KrnModelConnectionInput(indexIC,0 ,pnameIn0,ptypeIn0);
KrnModelConnectionInput(indexIC,1 ,pnameIn1,ptypeIn1);
KrnModelConnectionInput(indexIC,2 ,pnameIn2,ptypeIn2);
}
 break;

/*
 *        SYSTEM INITIALIZATION CODE 
 */
case SYSTEM_INIT:
     
	star_P->state_P = (char*)calloc(1,sizeof(state_t));
	state_P = (state_Pt)star_P->state_P;
     


         
   if(NO_OUTPUT_BUFFERS() != 3 ){
       fprintf(stdout,"%s:3 outputs expected; %d connected\n",
              STAR_NAME,NO_OUTPUT_BUFFERS());
	      return(201);
   }

   SET_CELL_SIZE_OUT(0,sizeof(float));

   SET_CELL_SIZE_OUT(1,sizeof(float));

   SET_CELL_SIZE_OUT(2,sizeof(float));

         
   if(NO_INPUT_BUFFERS() != 3 ){
       fprintf(stdout,"%s:3 inputs expected; %d connected\n",
              STAR_NAME,NO_INPUT_BUFFERS());
	      return(200);
   }

   SET_CELL_SIZE_IN(0,sizeof(float));

   SET_CELL_SIZE_IN(1,sizeof(float));

   SET_CELL_SIZE_IN(2,sizeof(float));

break;

/*
 *               USER INITIALIZATION CODE 
 */
case USER_INIT: 

 

	if ((samplesDelay >= 0) && (samplesDelay < 100))
	;
	else {
		fprintf(stderr,"cable: parameter out of range");
		return(1);
	}
	first=1;


break;

/* 
 *             MAIN CODE 
 */
case MAIN_CODE: 

 

if(first) {
	first=0;
	i=0;
	while ( i< samplesDelay) {
	if(IT_OUT(0)) {
		KrnOverflow("cable",0);
		return(99);
	}
	if(IT_OUT(1)) {
		KrnOverflow("cable",1);
		return(99);
	}
	if(IT_OUT(2)) {
		KrnOverflow("cable",2);
		return(99);
	}

	rxOut(0)=0.0;
	txOut(0)= 0.0;
	cdOut(0) = 0.0;
	++i;
	}
}

for(numberSamples = MIN_AVAIL(); numberSamples > 0; -- numberSamples) {
	IT_IN(0);
	IT_IN(1);
	IT_IN(2);


	if(IT_OUT(0)) {
		KrnOverflow("cable",0);
		return(99);
	}
	if(IT_OUT(1)) {
		KrnOverflow("cable",1);
		return(99);
	}
	if(IT_OUT(2)) {
		KrnOverflow("cable",2);
		return(99);
	}
	rxOut(0)=rxIn(0);
	txOut(0)=txIn(0);
	cdOut(0)=cdIn(0);
}
return(0);



break;

/*             
 *            WRAPUP CODE 
 */
case WRAPUP: 

 



break;
}
return(0);
}
