<BLOCK>
<LICENSE>
/*  Capsim (r) Text Mode Kernel (TMK) Star Library (Blocks)
    Copyright (C) 1989-2002  XCAD Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.xcad.com
    XCAD Corporation
    Raleigh, North Carolina */
</LICENSE>
<BLOCK_NAME>

firtaps 

</BLOCK_NAME> 

<COMMENTS>
<![CDATA[ 

/* firfil.s */
/***********************************************************************
                             firfil()
************************************************************************
This star designs FIR low pass, high pass, band pass, and band stop 
filters using the windowing method.
The star stores the specs of the FIR filter in the file tmp.spec.
The star stores the FIR filter taps in the file tmp.tap.
Date:  October 19, 1989 
Programmer: Sasan H. Ardalan. 
<NAME>
firfil
</NAME>
<DESCRIPTION>
This star designs FIR low pass, highj pass, band pass, and band stop 
filters using the windowing method.
The star stores the specs of the FIR filter in the file tmp.spec.
The star stores the FIR filter taps in the file tmp.tap.
</DESCRIPTION>
<PROGRAMMERS>
Date:  October 19, 1989 
Programmer: Sasan H. Ardalan. 
</PROGRAMMERS>
*/

]]>
</COMMENTS> 

<DESC_SHORT>
This block implements an  FIR based on taps stored in a file.
</DESC_SHORT>


<INCLUDES>
<![CDATA[ 

#include <math.h>
#include <stdio.h>

]]>
</INCLUDES> 

<DEFINES> 



</DEFINES> 

     

<STATES>
	<STATE>
		<TYPE>double*</TYPE>
		<NAME>x_P</NAME>
	</STATE>
	<STATE>
		<TYPE>double*</TYPE>
		<NAME>h_P</NAME>
	</STATE>
	<STATE>
		<TYPE>int</TYPE>
		<NAME>N</NAME>
	</STATE>
</STATES>
 
<DECLARATIONS> 

   	int i;
   	int j;
	int status;
	int numberTaps;
	float tmp1,tmp2;
        float sum;
	int	no_samples;
	FILE *imp_F;

</DECLARATIONS> 

                       

<PARAMETERS>
<PARAM>
	<DEF>File with Taps (first line # of taps)</DEF>
	<TYPE>file</TYPE>
	<NAME>fileName</NAME>
	<VALUE>tmp.tap</VALUE>
</PARAM>

</PARAMETERS>

    

<INPUT_BUFFERS>
	<BUFFER>
		<TYPE>float</TYPE>
		<NAME>x</NAME>
	</BUFFER>
</INPUT_BUFFERS>
 
    

<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE>float</TYPE>
		<NAME>y</NAME>
	</BUFFER>
</OUTPUT_BUFFERS>
 
<INIT_CODE>
<![CDATA[ 


	/*
	 * open file containing impulse response samples. Check 
	 * to see if it exists.
	 *
	 */
        if( (imp_F = fopen(fileName,"r")) == NULL) {
		fprintf(stderr,"firtaps: file could not be opened.\n");
		return(4);
	}
	fscanf(imp_F,"%d",&numberTaps);
	/*
	 * Allocate memory and return pointers for tapped delay line x_P and
	 * array containing impulse response samples, h_P.
	 *
	 */
	if( (x_P = (double*)calloc(numberTaps,sizeof(double))) == NULL ||
	    (h_P = (double*)calloc(numberTaps,sizeof(double))) == NULL ) {
	   	fprintf(stderr,"firtaps: can't allocate work space\n");
		return(4);
	}
	/*
	 * Read in the impulse response samples into the array
	 * and initialize the tapped delay line to zero.
	 *
	 */
	for (i=0; i<numberTaps; i++) {
		x_P[i]= 0.0;
		fscanf(imp_F,"%lf",&h_P[i]);
	}
	N=numberTaps;
	fclose(imp_F);

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 




	for(no_samples=MIN_AVAIL();no_samples >0; --no_samples) {
		IT_IN(0);
		/*
		 * Shift input sample into tapped delay line
		 */
		tmp2=x(0);
		for(i=0; i<N; i++) {
			tmp1=x_P[i];
			x_P[i]=tmp2;
			tmp2=tmp1;
		}
		/*
		 * Compute inner product 
		 */
                sum = 0.0;
		for (i=0; i<N; i++) { 
		     sum += x_P[i]*h_P[i];
		}
		if(IT_OUT(0)) {
			KrnOverflow("firfil",0);
			return(99);
		}
		/*
	  	 * set output buffer to response result
		 */
		y(0) = sum;
	}

]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 

	free(x_P); free(h_P); 

]]>
</WRAPUP_CODE> 



</BLOCK> 

