 
#ifdef LICENSE

/*  Capsim (r) Text Mode Kernel (TMK) Star Library (Blocks)
    Copyright (C) 1989-2002  XCAD Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.xcad.com
    XCAD Corporation
    Raleigh, North Carolina 
*/

#endif
 
#ifdef SHORT_DESCRIPTION

This star inputs an image and outputs multiple images representing color components.

#endif
 
#ifdef PROGRAMMERS

#endif






#include <stdio.h>
#include <math.h>

/* Note: these paths are installation dependent! */
#include <capsim.h>
#include <stars.h>



 

#include <math.h>
#include "dsp.h" 


 

#define RGB 	0
#define YCbCr	1
#define YUV	2
#define unsigned_short unsigned short


/*
 *           STATES STRUCTURE 
 */ 
typedef struct {
       unsigned_short   __red_A[256];
       unsigned_short   __green_A[256];
       unsigned_short   __blue_A[256];
     } state_t,*state_Pt;

/*
 *         STATE DEFINES 
 */ 
#define red_A (state_P->__red_A)
#define green_A (state_P->__green_A)
#define blue_A (state_P->__blue_A)

/*         
 *    INPUT BUFFER DEFINES 
 */ 
#define x(DELAY) (*(( image_t   *)PIN(0,DELAY)))

/*         
 *    OUTPUT BUFFER  DEFINES 
 */ 
#define y(delay) *( image_t   *)POUT(0,delay)

/*         
 *    PARAMETER DEFINES 
 */ 
#define colorSpace (param_P[0]->value.d)
/*-------------- BLOCK CODE ---------------*/

imgcolorsep
(run_state,block_P)

	int run_state;
	block_Pt block_P;
{
	param_Pt *param_P = block_P->param_AP;
	star_Pt star_P = block_P->star_P;

     
	state_Pt state_P = (state_Pt)star_P->state_P;
     
/*
 *              Declarations 
 */
 

	int no_samples;
	int i,j,k;
	float	temp;
	dsp_floatMatrix_t	matrix;
	dsp_floatMatrix_Pt	comp1_P;
	dsp_floatMatrix_Pt	comp3_P;
	dsp_floatMatrix_Pt	comp2_P;
	dsp_floatMatrix_Pt	subMatrix_P;
	int	pwidth;
	int	pheight;
	float**		mat_PP;
	image_t	img;
	int	pixel;
	int	yl,cr,cb,u,v;
	int	red,green,blue;


switch (run_state) {

 
 


 
case PARAM_INIT: 
    {
      int indexModel88 = block_P->model_index;
     char   *pdef0 = " Color Space:0=RGB,1=YCbCr,2=YUB ";
     char   *ptype0 = "int";
     char   *pval0 = "0";
     char   *pname0 = "colorSpace";
KrnModelParam(indexModel88,0 ,pdef0,ptype0,pval0,pname0);

      }
break;
   


/*         
 *    OUTPUT BUFFER SYSTEM  INITS 
 */ 
 case OUTPUT_BUFFER_INIT: 
 {
 int indexOC = block_P->model_index;
     char   *ptypeOut0 = " image_t ";
     char   *pnameOut0 = " y ";
KrnModelConnectionOutput(indexOC,0 ,pnameOut0,ptypeOut0);
}
 break;

/*         
 *    INPUT BUFFER SYSTEM  INITS 
 */ 
 case INPUT_BUFFER_INIT:
 {
 int indexIC = block_P->model_index;
     char   *ptypeIn0 = " image_t ";
     char   *pnameIn0 = " x ";
KrnModelConnectionInput(indexIC,0 ,pnameIn0,ptypeIn0);
}
 break;

/*
 *        SYSTEM INITIALIZATION CODE 
 */
case SYSTEM_INIT:
     
	star_P->state_P = (char*)calloc(1,sizeof(state_t));
	state_P = (state_Pt)star_P->state_P;
     


         
   if(NO_OUTPUT_BUFFERS() != 1 ){
       fprintf(stdout,"%s:1 outputs expected; %d connected\n",
              STAR_NAME,NO_OUTPUT_BUFFERS());
	      return(201);
   }

   SET_CELL_SIZE_OUT(0,sizeof( image_t ));

         
   if(NO_INPUT_BUFFERS() != 1 ){
       fprintf(stdout,"%s:1 inputs expected; %d connected\n",
              STAR_NAME,NO_INPUT_BUFFERS());
	      return(200);
   }

   SET_CELL_SIZE_IN(0,sizeof( image_t ));

break;

/*
 *               USER INITIALIZATION CODE 
 */
case USER_INIT: 

 

#if 0
	if(IIP_GetRGBColormap(red_A,green_A,blue_A,256)) {
		fprintf(stderr,"imgcolorsep: Problem getting RGB Values from colormap\n");
		return(1);
	}
#endif


break;

/* 
 *             MAIN CODE 
 */
case MAIN_CODE: 

 

/*
 * collect the image
 */
for (no_samples = MIN_AVAIL(); no_samples > 0; --no_samples) {
	IT_IN(0);
	img=x(0);
	pheight=img.height;
	pwidth=img.width;
	mat_PP=img.image_PP;

	if(IIP_GetRGBColormap(red_A,green_A,blue_A,256)) {
		fprintf(stderr,"imgcolorsep: Problem getting RGB Values from colormap\n");
		return(1);
	}

    	comp1_P=Dsp_AllocateMatrix(pwidth,pheight);
	comp2_P=Dsp_AllocateMatrix(pwidth,pheight);
	comp3_P=Dsp_AllocateMatrix(pwidth,pheight);

	if(comp1_P==NULL || comp3_P==NULL || comp2_P==NULL ) {
		fprintf(stderr,"imgcolorsep: allocation failure\n");
		return(2);
	}


	for(i=0; i<pheight; i++) {
		for(k=0; k<pwidth; k++) {
			pixel=(int)mat_PP[i][k];
			if(pixel <0 ) pixel=0;
			if(pixel > 255) pixel=255;
			red=red_A[pixel];
			green=green_A[pixel];
			blue=blue_A[pixel];

			switch(colorSpace) {
			  case RGB:
			      comp1_P->matrix_PP[i][k]=red;
			      comp2_P->matrix_PP[i][k]=green;
			      comp3_P->matrix_PP[i][k]=blue;
			      break;
			  case YCbCr:
			  case YUV:
			      yl=0.3*red+0.6*green+0.1*blue;
			      v=red-yl;
			      u=blue-yl;
			      cb=u/2+128;
			      cr=v/1.6+128;
			      cb = cb;
			      cr=  cr;
#if 1
			      if(cb > 255) cb=255;
			      if(cb > 255) cb=255;
#endif
			      if(cb <0) cb=0;
			      if(cr <0) cr=0;
#if 1
			      if(yl >255) yl=255;
			      if(u > 255) u=255;
			      if(v > 255) v=255;
#endif
			      if(yl <0) yl=0;
			      if(u <0) u=0;
			      if(v <0) v=0;
			      if(colorSpace == YUV) {
			        comp1_P->matrix_PP[i][k]=yl;
			        comp2_P->matrix_PP[i][k]=u;
			        comp3_P->matrix_PP[i][k]=v;
			      } else {
			        comp1_P->matrix_PP[i][k]=yl;
			        comp2_P->matrix_PP[i][k]=cb;
			        comp3_P->matrix_PP[i][k]=cr;
			      }
				
			}
		}
	} 
	/*
	 * output image components 
	 */
        if(IT_OUT(0) ){ KrnOverflow("imgcolorsep",0); return(99); }

	img.image_PP=comp1_P->matrix_PP;
	img.width=comp1_P->width;
	img.height=comp1_P->height;

	y(0) = img;
	/*
	 * output green image
	 */
        if(IT_OUT(0) ){ KrnOverflow("imgcolorsep",0); return(99); }

	img.image_PP= comp2_P->matrix_PP;
	img.width= comp2_P->width;
	img.height= comp2_P->height;

	y(0) = img;
	/*
	 * output blue image
	 */
        if(IT_OUT(0) ){ KrnOverflow("imgcolorsep",0); return(99); }

	img.image_PP=comp3_P->matrix_PP;
	img.width=comp3_P->width;
	img.height=comp3_P->height;

	y(0) = img;
			
}
return(0);


break;

/*             
 *            WRAPUP CODE 
 */
case WRAPUP: 

 



break;
}
return(0);
}
