<BLOCK>
<LICENSE>
/*  Capsim (r) Text Mode Kernel (TMK) Star Library (Blocks)
    Copyright (C) 1989-2002  XCAD Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.xcad.com
    XCAD Corporation
    Raleigh, North Carolina 
*/
</LICENSE>
<BLOCK_NAME>
imgfdct
</BLOCK_NAME> 


<DESC_SHORT>
This star inputs an image and computes the DCT. Note that the input images must be 8x8.
</DESC_SHORT>


<COMMENTS>
<![CDATA[ 

/* imgfdtc.s */
/***********************************************************************
                             imgfdtc()
************************************************************************
This star inputs an image and computes the DCT.
Note that the input images must be 8x8.
If not an error message is printed and the star returns.
The DCT images are sequentially output.
Programmer:  	Sasan Ardalan	
Date: 		August 15, 1993
*/
]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 

#include <math.h>
#include "dsp.h" 

]]>
</INCLUDES> 

<DECLARATIONS> 

	int no_samples;
	int i,j,k;
	float	temp;
	dsp_floatMatrix_t	matrix;
	dsp_floatMatrix_Pt	matrix_P;
	dsp_floatMatrix_Pt	subMatrix_P;
	int	pwidth;
	int	pheight;
	float**		mat_PP;
	image_t	img;

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> short* </TYPE>
		<NAME> dctBlock_A </NAME>
	</STATE>
</STATES>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> image_t </TYPE>
		<NAME> x </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> image_t </TYPE>
		<NAME> y </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 

	dctBlock_A=(short*)calloc(64,sizeof(short));

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 

for (no_samples = MIN_AVAIL(); no_samples > 0; --no_samples) {
	/*
	 * collect the image
	 */
	IT_IN(0);
	img=x(0);
	pheight=img.height;
	pwidth=img.width;
	mat_PP=img.image_PP;

	if(!(pheight == 8 && pwidth == 8)) {
		fprintf(stderr,"Forward DCT: image must be 8x8\n");
		return(4);
	}

	/*
	 * put image into dct block
	 * note that we convert 0-255 range tp -128 to 128 range
	 */
	k=0;
	for(i=0; i<pheight; i++)
		for(j=0; j<pwidth; j++) {
			dctBlock_A[k]=mat_PP[i][j]-128;
			k++;
		}
	/*
	 * perform 8x8 Forward DCT
	 */
	j_fwd_dct(dctBlock_A);
	/*
	 * put dct block into image 
	 */
	k=0;
	for(i=0; i<pheight; i++)
		for(j=0; j<pwidth; j++) {
			mat_PP[i][j]=dctBlock_A[k];
			k++;
		}
	/*
	 * Output image
	 */

        if(IT_OUT(0) ){
		    KrnOverflow("imgfdtc",0);
		    return(99);
        }


        y(0) = img;
			
}
return(0);

]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 


]]>
</WRAPUP_CODE> 



</BLOCK> 

