 
#ifdef LICENSE

/*
 * (c) XCAD  Corporation, Portland, Oregon, All Rights Reserved
 * Acknowledgemnet: Dr. Yoon.
 */



#endif
 
#ifdef SHORT_DESCRIPTION

Model a nonlinear Power Amplifier specifying 1dB compression, gain and backoff. Real input/output.

#endif
 
#ifdef PROGRAMMERS

#endif






#include <stdio.h>
#include <math.h>

/* Note: these paths are installation dependent! */
#include <capsim.h>
#include <stars.h>



 




/*
 *           STATES STRUCTURE 
 */ 
typedef struct {
       float   __sumOut;
       float   __aGain;
       float   __bGain;
       float   __cmp;
       float   __rout;
       float   __rin;
       float   __bov;
       float   __rhoInMax;
       float   __rhoOutMax;
       int   __counter;
     } state_t,*state_Pt;

/*
 *         STATE DEFINES 
 */ 
#define  sumOut  (state_P->__sumOut)
#define  aGain  (state_P->__aGain)
#define  bGain  (state_P->__bGain)
#define  cmp  (state_P->__cmp)
#define  rout  (state_P->__rout)
#define  rin  (state_P->__rin)
#define  bov  (state_P->__bov)
#define  rhoInMax  (state_P->__rhoInMax)
#define  rhoOutMax  (state_P->__rhoOutMax)
#define  counter  (state_P->__counter)

/*         
 *    INPUT BUFFER DEFINES 
 */ 
#define inStream(DELAY) (*(( float   *)PIN(0,DELAY)))

/*         
 *    OUTPUT BUFFER  DEFINES 
 */ 
#define outStream(delay) *( float   *)POUT(0,delay)

/*         
 *    PARAMETER DEFINES 
 */ 
#define out1dB (param_P[0]->value.f)
#define G (param_P[1]->value.f)
#define sourceR (param_P[2]->value.f)
#define s (param_P[3]->value.f)
#define bo (param_P[4]->value.f)
#define windowLength (param_P[5]->value.d)
/*-------------- BLOCK CODE ---------------*/

patx
(run_state,block_P)

	int run_state;
	block_Pt block_P;
{
	param_Pt *param_P = block_P->param_AP;
	star_Pt star_P = block_P->star_P;

     
	state_Pt state_P = (state_Pt)star_P->state_P;
     
/*
 *              Declarations 
 */
 

  float rho, avgPower, pNat,outPowerDBm;
  double tmpIn;
  double thirdOrder;
  double tmpRe, tmpIm;
  double tmpOutRe, tmpOutIm;


switch (run_state) {

 
 


 
case PARAM_INIT: 
    {
      int indexModel88 = block_P->model_index;
     char   *pdef0 = " Output 1-db compression point(dBm) ";
     char   *ptype0 = "float";
     char   *pval0 = "25";
     char   *pname0 = "out1dB";
     char   *pdef1 = " Linear Gain(dB) ";
     char   *ptype1 = "float";
     char   *pval1 = "30";
     char   *pname1 = "G";
     char   *pdef2 = " Source Impedance ";
     char   *ptype2 = "float";
     char   *pval2 = "50";
     char   *pname2 = "sourceR";
     char   *pdef3 = " Knee sharpness of limiter ";
     char   *ptype3 = "float";
     char   *pval3 = "1";
     char   *pname3 = "s";
     char   *pdef4 = " Input Backoff ";
     char   *ptype4 = "float";
     char   *pval4 = "-6";
     char   *pname4 = "bo";
     char   *pdef5 = " Window length for power estimate ";
     char   *ptype5 = "int";
     char   *pval5 = "25";
     char   *pname5 = "windowLength";
KrnModelParam(indexModel88,0 ,pdef0,ptype0,pval0,pname0);
KrnModelParam(indexModel88,1 ,pdef1,ptype1,pval1,pname1);
KrnModelParam(indexModel88,2 ,pdef2,ptype2,pval2,pname2);
KrnModelParam(indexModel88,3 ,pdef3,ptype3,pval3,pname3);
KrnModelParam(indexModel88,4 ,pdef4,ptype4,pval4,pname4);
KrnModelParam(indexModel88,5 ,pdef5,ptype5,pval5,pname5);

      }
break;
   


/*         
 *    OUTPUT BUFFER SYSTEM  INITS 
 */ 
 case OUTPUT_BUFFER_INIT: 
 {
 int indexOC = block_P->model_index;
     char   *ptypeOut0 = " float ";
     char   *pnameOut0 = " outStream ";
KrnModelConnectionOutput(indexOC,0 ,pnameOut0,ptypeOut0);
}
 break;

/*         
 *    INPUT BUFFER SYSTEM  INITS 
 */ 
 case INPUT_BUFFER_INIT:
 {
 int indexIC = block_P->model_index;
     char   *ptypeIn0 = " float ";
     char   *pnameIn0 = " inStream ";
KrnModelConnectionInput(indexIC,0 ,pnameIn0,ptypeIn0);
}
 break;

/*
 *        SYSTEM INITIALIZATION CODE 
 */
case SYSTEM_INIT:
     
	star_P->state_P = (char*)calloc(1,sizeof(state_t));
	state_P = (state_Pt)star_P->state_P;
     


         
   if(NO_OUTPUT_BUFFERS() != 1 ){
       fprintf(stdout,"%s:1 outputs expected; %d connected\n",
              STAR_NAME,NO_OUTPUT_BUFFERS());
	      return(201);
   }

   SET_CELL_SIZE_OUT(0,sizeof( float ));

         
   if(NO_INPUT_BUFFERS() != 1 ){
       fprintf(stdout,"%s:1 inputs expected; %d connected\n",
              STAR_NAME,NO_INPUT_BUFFERS());
	      return(200);
   }

   SET_CELL_SIZE_IN(0,sizeof( float ));

break;

/*
 *               USER INITIALIZATION CODE 
 */
case USER_INIT: 

 

    rout = sourceR;
    rin = sourceR;
    /* 
     * Calculate voltage gain from power gain 
     */
    aGain = (pow(10.0, (G/20.0)));
    /* 
     * Calculate 1dB compression in voltage from dBm 
     */
    cmp = sqrt(pow(10.0, out1dB/10)*2*rout*0.001)/aGain;
    printf("cmp=%f\n",cmp);
    /*
     * Scale input for input backoff 
     */
    bov=(pow(10.0, (bo/20.0)));
    
    /* 
     * Calculate third order term 
     */
    bGain = (0.108749)/(cmp*cmp);
    /*
     * Calculate soft-limiting input threshold 
     */
    rhoInMax = cmp*s;
    printf("rhoInMax=%f\n",rhoInMax);
    /*
     * Calculate output limiting ceiling 
     */
    rhoOutMax = aGain*s*cmp*(1-bGain*pow((cmp*s),2));
    printf("rhoOutMax=%f\n",rhoOutMax);
    counter = 0;
    sumOut = 0.0;
 

break;

/* 
 *             MAIN CODE 
 */
case MAIN_CODE: 

 


  
while( AVAIL(0) ) {
    IT_IN(0);
    counter++;
    tmpIn = inStream(0);

    tmpIn *= bov;
   
    rho=fabs(tmpIn);

    if (rho < rhoInMax) {
	  /* third_order(y) = in(y)*a*(1 - b*in(y)^2); */
	  
	  tmpRe=tmpIn*tmpIn;
	  


	  tmpRe = aGain*(1.0 - bGain*tmpRe);
	  
	  
	  
	  tmpOutRe=tmpIn*tmpRe;
	   
			 
			 
			 
			 
	  thirdOrder =  tmpOutRe;
         
    }
    else {
	  /* third_order(y) = rhoOutMax*sign(in(y)); */
	  thirdOrder = rhoOutMax * tmpIn/rho;
	  
    }

    if (IT_OUT(0)){
	   KrnOverflow("patx",0);
	   return(99);
    }

    outStream(0) = thirdOrder;

    /* Estimate Output Power */
     
    sumOut += thirdOrder*thirdOrder;
    if(counter == windowLength) {

          pNat = sumOut/(float)windowLength;
          outPowerDBm=10*log10(10*pNat);
          printf("OBO=%f\n",(out1dB - outPowerDBm));
    }

	
}


break;

/*             
 *            WRAPUP CODE 
 */
case WRAPUP: 

 



break;
}
return(0);
}
