<BLOCK>
<LICENSE>
/*
 * (c) XCAD  Corporation, Portland, Oregon, All Rights Reserved
 * Acknowledgemnet: Dr. Yoon.
 */


</LICENSE>
<BLOCK_NAME>
patx
</BLOCK_NAME> 

             
<DESC_SHORT>
Model a nonlinear Power Amplifier specifying 1dB compression, gain and backoff. Real input/output.
</DESC_SHORT>


<COMMENTS>
<![CDATA[ 

]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 


]]>
</INCLUDES> 

<DECLARATIONS> 

  float rho, avgPower, pNat,outPowerDBm;
  double tmpIn;
  double thirdOrder;
  double tmpRe, tmpIm;
  double tmpOutRe, tmpOutIm;

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> sumOut </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> aGain </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> bGain </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> cmp </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> rout </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> rin </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> bov </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> rhoInMax </NAME>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> rhoOutMax </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> counter </NAME>
	</STATE>
</STATES>



<PARAMETERS>
	<PARAM>
		<DEF> Output 1-db compression point(dBm) </DEF>
		<TYPE> float </TYPE>
		<NAME> out1dB </NAME>
		<VALUE> 25 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Linear Gain(dB) </DEF>
		<TYPE> float </TYPE>
		<NAME> G </NAME>
		<VALUE> 30 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Source Impedance </DEF>
		<TYPE> float </TYPE>
		<NAME> sourceR </NAME>
		<VALUE> 50 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Knee sharpness of limiter </DEF>
		<TYPE> float </TYPE>
		<NAME> s </NAME>
		<VALUE> 1 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Input Backoff </DEF>
		<TYPE> float </TYPE>
		<NAME> bo </NAME>
		<VALUE> -6 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Window length for power estimate </DEF>
		<TYPE> int </TYPE>
		<NAME> windowLength </NAME>
		<VALUE> 25 </VALUE>
	</PARAM>
</PARAMETERS>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> inStream </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> outStream </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 

    rout = sourceR;
    rin = sourceR;
    /* 
     * Calculate voltage gain from power gain 
     */
    aGain = (pow(10.0, (G/20.0)));
    /* 
     * Calculate 1dB compression in voltage from dBm 
     */
    cmp = sqrt(pow(10.0, out1dB/10)*2*rout*0.001)/aGain;
    printf("cmp=%f\n",cmp);
    /*
     * Scale input for input backoff 
     */
    bov=(pow(10.0, (bo/20.0)));
    
    /* 
     * Calculate third order term 
     */
    bGain = (0.108749)/(cmp*cmp);
    /*
     * Calculate soft-limiting input threshold 
     */
    rhoInMax = cmp*s;
    printf("rhoInMax=%f\n",rhoInMax);
    /*
     * Calculate output limiting ceiling 
     */
    rhoOutMax = aGain*s*cmp*(1-bGain*pow((cmp*s),2));
    printf("rhoOutMax=%f\n",rhoOutMax);
    counter = 0;
    sumOut = 0.0;
 
]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


  
while( AVAIL(0) ) {
    IT_IN(0);
    counter++;
    tmpIn = inStream(0);

    tmpIn *= bov;
   
    rho=fabs(tmpIn);

    if (rho < rhoInMax) {
	  /* third_order(y) = in(y)*a*(1 - b*in(y)^2); */
	  
	  tmpRe=tmpIn*tmpIn;
	  


	  tmpRe = aGain*(1.0 - bGain*tmpRe);
	  
	  
	  
	  tmpOutRe=tmpIn*tmpRe;
	   
			 
			 
			 
			 
	  thirdOrder =  tmpOutRe;
         
    }
    else {
	  /* third_order(y) = rhoOutMax*sign(in(y)); */
	  thirdOrder = rhoOutMax * tmpIn/rho;
	  
    }

    if (IT_OUT(0)){
	   KrnOverflow("patx",0);
	   return(99);
    }

    outStream(0) = thirdOrder;

    /* Estimate Output Power */
     
    sumOut += thirdOrder*thirdOrder;
    if(counter == windowLength) {

          pNat = sumOut/(float)windowLength;
          outPowerDBm=10*log10(10*pNat);
          printf("OBO=%f\n",(out1dB - outPowerDBm));
    }

	
}

]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 


]]>
</WRAPUP_CODE> 



</BLOCK> 

