<BLOCK>
<LICENSE>
/*  Capsim (r) Text Mode Kernel (TMK) Star Library (Blocks)
    Copyright (C) 1989-2002  XCAD Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.xcad.com
    XCAD Corporation
    Raleigh, North Carolina 
*/
</LICENSE>
<BLOCK_NAME>
spectrogram
</BLOCK_NAME> 

<DESC_SHORT>
This routine produces the spectrogram of the input buffer.
</DESC_SHORT>


<COMMENTS>
<![CDATA[ 

/* spectrogram.s */
/**********************************************************************
                                spectrogram()
***********************************************************************
        inputs:         (One channel)
        outputs:        (optional feed-through of input channels)
*************************************************************************
This routine produces the spectrogram 
of the input buffer.
Programmer: 	Sasan Ardalan
Date: 		Dec. 4, 1993	
*/
]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 

#include "../XBLOCKS/iiplot.h"

]]>
</INCLUDES> 

<DEFINES> 

#define FLOAT_BUFFER 0
#define INTEGER_BUFFER 1
#define RECTANGULAR_WINDOW 0
#define HAMMING_WINDOW 1
#define PI 3.1415926535898 
#define BLOCK_SIZE 512

</DEFINES> 

<DECLARATIONS> 

        iip_Curve_Pt     curveTime_P;
        iip_Curve_Pt     curve_P;
	int samples;
	int numberPoints;
    	int i,j,ii;
	float tmp;
	complex val;
	float wind;
	char title1[80];
	char fname[80];
        char curveSubTitle[80];
	float* mag_P;
	float* phase_P;
	FILE *time_F;
	FILE *freq_F;

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> numberInputBuffers </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> numberOutputBuffers </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> xpts </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> xTime_P </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> ypts </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> spect_P </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> count </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> totalCount </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> blockOff </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> bufi </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> float </TYPE>
		<NAME> dx </NAME>
	</STATE>
</STATES>



<PARAMETERS>
	<PARAM>
		<DEF> Number of points </DEF>
		<TYPE> int </TYPE>
		<NAME> npts </NAME>
		<VALUE> 128  </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Number of points to skip </DEF>
		<TYPE> int </TYPE>
		<NAME> skip </NAME>
		<VALUE> 0  </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Window size (samples) </DEF>
		<TYPE> int </TYPE>
		<NAME> windowLength </NAME>
		<VALUE> 128  </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Overlap (samples) </DEF>
		<TYPE> int </TYPE>
		<NAME> overlap </NAME>
		<VALUE> 0  </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Zero padding (samples) </DEF>
		<TYPE> int </TYPE>
		<NAME> zeropad </NAME>
		<VALUE> 0  </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Threshold (DB) </DEF>
		<TYPE> float </TYPE>
		<NAME> threshold </NAME>
		<VALUE> 40 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Sampling Rate (KHz) </DEF>
		<TYPE> float </TYPE>
		<NAME> samplingRate </NAME>
		<VALUE> 8 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Window:0= Rec.,1=Hamming </DEF>
		<TYPE> int </TYPE>
		<NAME> windowType </NAME>
		<VALUE> 1 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Linear = 0, dB = 1 </DEF>
		<TYPE> int </TYPE>
		<NAME> dBFlag </NAME>
		<VALUE> 1 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Surface Plot: 1=True 0=False </DEF>
		<TYPE> int </TYPE>
		<NAME> surfaceFlag </NAME>
		<VALUE> 0 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Image: 1=True 0=False </DEF>
		<TYPE> int </TYPE>
		<NAME> imageFlag </NAME>
		<VALUE> 1 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Plot title </DEF>
		<TYPE> file </TYPE>
		<NAME> spectTitle </NAME>
		<VALUE> Spectrogram </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Time Domain On/Off (1/0) </DEF>
		<TYPE> int </TYPE>
		<NAME> timeFlag </NAME>
		<VALUE> 1 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Control: 1=On, 0=Off </DEF>
		<TYPE> int </TYPE>
		<NAME> control </NAME>
		<VALUE> 1 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Buffer type:0= Float,1=Integer </DEF>
		<TYPE> int </TYPE>
		<NAME> bufferType </NAME>
		<VALUE> 0 </VALUE>
	</PARAM>
</PARAMETERS>

<INIT_CODE>
<![CDATA[ 

/* store as state the number of input/output buffers */
if((numberInputBuffers = NO_INPUT_BUFFERS()) <= 0) {
	fprintf(stderr,"spectrogram: no inputs connected\n");
	return(1);
}
if(numberInputBuffers > 1) {
	fprintf(stderr,"spectrogram: only one  input allowed \n");
	return(2);
}
if((numberOutputBuffers = NO_OUTPUT_BUFFERS()) > numberInputBuffers) {
	fprintf(stderr,"spectrogram: too many outputs connected\n");
	return(3);
}
if(samplingRate)
	dx=1.0/samplingRate;
else
	dx=1.0;
if (control) {
	/*                                                      
	 * allocate arrays                                      
	 */                                                     
	xpts = (float* )calloc(BLOCK_SIZE,sizeof(float));           
	ypts = (float* )calloc(BLOCK_SIZE,sizeof(float));           
	if(xpts==NULL  || ypts ==NULL) {
		fprintf(stderr,"Spectrogram could not allocate space\n");
		return(4);
	}
}
count = 0;
totalCount = 0;
switch(bufferType) {
	case FLOAT_BUFFER: 
		SET_CELL_SIZE_IN(0,sizeof(float));
		if(numberOutputBuffers == 1)
			SET_CELL_SIZE_OUT(0,sizeof(float));
		break;
	case INTEGER_BUFFER: 
		SET_CELL_SIZE_IN(0,sizeof(int));
		if(numberOutputBuffers == 1)
			SET_CELL_SIZE_OUT(0,sizeof(int));
		break;
	default: 
		fprintf(stderr,"Bad buffer type specified in spectrogram \n");
		return(5);
		break;
}

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


for(samples = MIN_AVAIL(); samples > 0; --samples) {

  for(i=0; i<numberInputBuffers; ++i) {
   		IT_IN(i);
		if(numberOutputBuffers > i) {
			if(IT_OUT(i)) {
				KrnOverflow("spectrogram",i);
				return(99);
			}

			switch(bufferType) {
				case INTEGER_BUFFER:
	 				OUTI(i,0) = INI(i,0);
					break;
				case FLOAT_BUFFER:
	 				OUTF(i,0) = INF(i,0);
					break;
			}

		}
  }
  if(++totalCount > skip && control ) {
		count=blockOff + bufi;
		bufi++;
		if (bufi == BLOCK_SIZE) {
			blockOff += BLOCK_SIZE;
			xpts = (float *)realloc((char *) xpts, 
				sizeof(float) * (blockOff + BLOCK_SIZE));
			ypts = (float *)realloc((char *) ypts, 
				sizeof(float) * (blockOff + BLOCK_SIZE));
			bufi=0;
		}
		if(xpts == NULL || ypts==NULL) {
			fprintf(stderr,"spectrogram: could not allocate space\n");
			return(6);
		}

		switch(bufferType) {
			case INTEGER_BUFFER:
          	     		ypts[count] = INI(0,0);
				break;
			case FLOAT_BUFFER:
          	     		ypts[count] = INF(0,0);
				break;
		}
		xpts[count]=count*dx;

		
  }

} 

return(0);


]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 

if(control == 0) return(0);
if((totalCount - skip) > 0 ) {
   /*
    * callup the plot routine 
    */
   if(iip_hdr_P ) {
	/*
	 * if in graphic mode callup iip
	 */
	 xpts = (float *) realloc((char *) xpts, sizeof(float) *
				 (blockOff + bufi));
	 ypts = (float *) realloc((char *) ypts, sizeof(float) *
				 (blockOff + bufi));
	if(xpts == NULL || ypts==NULL) {
		fprintf(stderr,"spectrogram: could not allocate space\n");
		return(6);
	}
	curveTime_P = IIP_NewInitedCurveRec();
	curveTime_P->qpCurveFlag = TRUE;
	curveTime_P->x_P = xpts;
	curveTime_P->y_P = ypts;
	curveTime_P->points = count;
	curveTime_P->note_A = NULL;
        curveTime_P->funcType = PLT_GROUP_TIME_DOMAIN;
	sprintf(title1,"%s Time Domain %s",spectTitle,SNAME(0));
	IIP_SetTitles(curveTime_P,title1,NULL);
	if(timeFlag)
		IIP_AppendCurve(curveTime_P);
		curve_P=IIP_Spectrogram(curveTime_P,windowLength,overlap,
				zeropad,
				samplingRate, windowType,threshold,surfaceFlag,
				imageFlag,dBFlag,0,NULL);
	if(curve_P)
		strcpy(curve_P->title,spectTitle);
   }
}

]]>
</WRAPUP_CODE> 



</BLOCK> 

