.po +0.5i
.nr pp 12
.nr sp 12
.nr tp 12
.nr qp 12
.nr fp 10
.fo ''%''
.(b C
.ps 14
.b "BLOSIM 3.0 Reference Manual"
.ps 12
.sp 2
David J. Hait
David G. Messerschmitt
Department of Electrical Engineering and Computer Science
University of California
Berkeley, California 94720

Modifications, corrections and description of upgrades:
L. James Faber
ECE Department
North Carolina State University
Raleigh, NC 27695
.sp 2
\*(td
.)b
.he 'BLOSIM 3.6 Reference Manual''\*(td'
.sp 2
.(f
The initial Blosim development was supported by
the National Science Foundation, under Grant ECS-82-11071.
.)f
.sh 1 "Introduction"
.lp
BLOSIM (for BLOck SIMulator) is a program
for the functional simulation of communications and
signal processing systems.
In BLOSIM, simulations are constructed out of a 
variety of building blocks.
Each building block, known as a \fIstar\fR,
represents a small segment of object code
that executes a part of the simulation.
Stars are linked together by specifying the flow of data between them,
in the same way that you might wire
together individual components in an
actual system.
Once the system is built,
BLOSIM can then simulate it by
executing each star independently,
and transfering data samples between them as needed.
.lp
Stars can also be combined into larger, more complicated blocks.
Once the structure of these larger blocks,
which are called \fIgalaxies\fR, is defined, they can be 
interconnected in the same way as stars.
This approach to simulation allows you to
simulate large, complicated systems
without much of the overhead associated with
single-application simulation programming. Also, because of the 
standardized nature of each block, simulation code written by different
programmers can be effortlessly tied together.
.lp
This reference manual gives a complete list of BLOSIM's capabilities,
and thus is not intended as an introductory guide.  If
you are not familiar with BLOSIM, it is suggested that
you read \fIA BLOSIM Tutorial\fR [1]
prior to reading this manual.
.sh 1 "The BLOSIM Environment"
.lp
As we mentioned before,
a BLOSIM simulation consists entirely of blocks
and their interconnections.
A block can be a \fIstar\fR, in which case it refers to
simulation programming,  or 
it can be a \fIgalaxy\fR,  which means that the block's
function is described in terms of more primitive blocks
and the interconnections between them.  For example, a 
digital filter can be implemented as a star by writing
simulation programming for it, or as a galaxy by specifying
an interconnection of multipliers, adders, and delays, each
of which is a star.
.lp
Stars in BLOSIM always refer to a file
that contains the programming for the star. The file
may be either compiled object code, a C program in
the proper format, or STARGAZE source code [2].
The name of this file must be specified when the star
is created.
In addition, before a star can be executed
during a simulation,
one copy of the compiled code must be
loaded into memory.  BLOSIM sets up the
private data structures that allow many stars to share
this one copy.
.lp
Included with BLOSIM is small 
standard library of commonly used star object modules.
This library
has already been linked to the object code for BLOSIM and is
automatically loaded into memory when BLOSIM starts up.
Stars whose object modules are part of the standard library
are known as \fIlibrary stars\fR.  Since library stars don't
actually reference files, you specify a \fImodule\fR name
rather than a file name when you create a library star.
.lp
While each star refers to a simulation program segment,
every galaxy has associated with it a specification of
what blocks it contains and how they
are interconnected.  This specification is known in
BLOSIM as a \fItopology\fR, and is stored in
a \fItopology file\fR.  As with stars,  the topology
file for a galaxy must be specified at the time 
the galaxy is created.
.lp
A galaxy can be built out of other galaxies as well as stars.
In this way, we can consider every block in BLOSIM to have a place
in a hierarchy.  This hierarchy can be represented as
a tree structure,  with each node of the tree being 
a BLOSIM block
(a star or a galaxy).  If a node is a galaxy, it has a
number of child nodes, each of which corresponds to
a block that is part of that galaxy.  At the leaves of
the tree are the stars, which have no children.  Thus,
each block in BLOSIM has a \fIparent galaxy\fR, that is,
the galaxy of which it is a part.
.lp
At the very top, or root,  of the tree
is a node whose children are the top-level
blocks of the simulation. This node is 
called the \fIuniverse\fR. It is a galaxy, because
like any other galaxy, it contains a number of
blocks.  Unlike any other galaxy, however, the universe
has no input or output connections.  In addition, the 
universe does not have to be explicitly created
by the user; it is automatically created by BLOSIM
when the system starts up, and is initialized to 
be empty.
.lp
BLOSIM allows you to display and edit only one galaxy at
a time, known as the \fIcurrent galaxy\fR.  When
BLOSIM starts up, the current galaxy is the
universe.  By using the \fBup\fR and \fBdown\fR commands
(see below), you can move up and down the BLOSIM 
hierarchy, displaying and changing blocks at different
levels independently.
.lp
Just as each galaxy has a topology file associated with it,
the universe has its own topology file. This universe
topology file specifies which blocks are to be included at
the top level of the BLOSIM simulation, and
how they are to be interconnected.  The default
universe topology file is named \*(lquniverse.t\*(rq.
.sh 1 "Invoking Blosim"
.lp
To start up BLOSIM, type the command \fBblosim\fR from the Shell: 
.(b
% \fBblosim\fR
.)b
Invoked in this way, BLOSIM works interactively. If instead, you
specify a file,
.(b
% \fBblosim \fIfile\fB.t\fR
.)b
BLOSIM will run non-interactively, by
loading the file as the universe topology,
executing a \fBrun\fR (see below), and exiting.
Note that the file name ends in \*(lq.t\*(rq.
User entry of this suffix is optional;
any other suffix is stripped and a \*(lq.t\*(rq is substituted.
.lp
Other command options to BLOSIM are:
.(b
% \fBblosim -l \fIfile\fB.t\fR
.)b
Load the specified topology file,
and then prompt for interactive commands.
This file is also used by the 
\fBload\fR and \fBstore\fR commands (see below).
.lp 
When BLOSIM starts up, it will check the current directory, and then
your home directory, for a file named \*(lq.blosimrc\*(rq.
This file, if it exists, must contain valid BLOSIM commands,
one per line.
BLOSIM will read this file and execute the commands in it
before displaying the prompt
(or before loading and executing the specified topology,
if invoked non-interactively).  
If any command in the \*(lq.blosimrc\*(rq file is in error,
BLOSIM will terminate prematurely.
If BLOSIM is invoked with the -l option,
the \*(lq.blosimrc\*(rq commands are read
\fIbefore\fR the -l file is loaded.
.lp
To leave BLOSIM, the \fBexit\fR command or the \fBquit\fR
command work identically:
.(b
Blosim> \fBexit\fR
Blosim> \fBquit\fR
.)b
\fBWarning: neither command checks to see if the topology
of the universe
has been saved.\fR
.sh 1 "Creating Stars"
.lp
The \fBstar\fR command is used to create a star and place it in the
current parent galaxy.  The first argument to this command is
always the name of the star to be created.  A star may have a
name that is 
any alphanumeric string of reasonable length. No two
blocks in the same galaxy (or universe) may have the same name.
.lp
The second argument to \fBstar\fR tells where to get the object
code for this star.  If the suffix of this argument is \*(lq.s\*(rq,
the programming for the star is in a STARGAZE file [2].  If the
suffix is \*(lq.c\*(rq, the programming is in a C language source
file.  \*(lq.o\*(rq means the programming is in an object code
file.  If there is no suffix, BLOSIM assumes that the second
argument is the name of a library module.  In cases where
the argument refers to a file, rather than a library module,
BLOSIM checks to see whether the object code for the
star is currently resident in memory.  If not, BLOSIM
searches through the star path search list (defaults to 
the current directory; see \fBpath\fR). 
Source files are recompiled according to the following
rules:
.np
If a \*(lq.s\*(rq file is specified, STARGAZE and the C compiler
are always run to generate a new \*(lq.o\*(rq file.
.np
If a \*(lq.c\*(rq file is specified, and BLOSIM can't find it,
it searches for a \*(lq.s\*(rq file that STARGAZE can use
to generate it.
STARGAZE is also run if a \*(lq.s\*(rq file
exists in the same directory as the specified \*(lq.c\*(rq file,
and has been modified since the \*(lq.c\*(rq file was created.
The C compiler is always run on the \*(lq.c\*(rq file
to generate a \*(lq.o\*(rq file.
.np
If a \*(lq.o\*(rq file is specified, and is located, the
same directory is checked for
\*(lq.c\*(rq and \*(lq.s\*(rq  with the same name (except for the
suffix).  If these files exist, the \*(lq.o\*(rq file is 
recompiled if it is out-of-date.  If the \*(lq.o\*(rq file
can not be located, BLOSIM looks for
the correct \*(lq.s\*(rq or \*(lq.c\*(rq files, and uses
one of these to generate the \*(lq.o\*(rq file.
.lp
Once the \*(lq.o\*(rq file is located, it is loaded, and the star
is created.
.lp
Sometime it may be more convenient for you to load in the object
code for a star without actually creating a new instance
of that star.  BLOSIM allows you to do this with the \fBdefine\fR
command:
.(b
Blosim> \fBdefine s \fIfile\fR
.)b
BLOSIM will locate or create the object code for the specified
file in the same way that it would for the \fBstar\fR command. 
BLOSIM then loads this code, without creating a star.
.sh 1 "Creating Galaxies"
.lp
To create a galaxy, use the \fBgalaxy\fR command:
.(b
Blosim> \fBgalaxy\fI name topfile\fB.t\fR
.)b
The galaxy's name is \fIname\fR; the same rules apply as for
stars.  \fITopfile\fB.t\fR is a topology file (the \*(lq.t\*(rq
suffix is optional, but used in the file search).
.lp
The \fBgalaxy\fR command searches for the specified topology
file in the directories in the galaxy path search list
(see \fBpath\fR below).  Once found, that file
is used to create the internal structure of the new
galaxy.  Since new stars and galaxies may be part of this
structure, the \fBgalaxy\fR command may have to load in many
other topologies or star object code, and thus may
take a few seconds to complete.
.lp
If any errors are found in either the specified topology
file or the topology file of one of the galaxy's children,
a brief error message is printed, with the name of the
offending file, and the line number where the error 
occurred.  The galaxy whose topology file generated the error
is cleared of all blocks.
A simulation will not run when the universe contains empty galaxies.
A galaxy can become filled again if it is rebuilt from within BLOSIM,
or when its file is fixed and it is reloaded.
.lp
The \fBdefine\fR command can also be used for galaxies:
.(b
Blosim> \fBdefine g\fI file\fR
.)b
In this case, however, BLOSIM merely locates the specified topology
file (whose name must end in \fB.t\fR); its contents are not loaded.
.sh 1 "Parameters"
.lp
A parameter is a variable argument to a star or a galaxy
that can be set from within BLOSIM prior to running a simulation.
Stars access the values of their parameters through special subroutine
calls  (these calls are part of the programming automatically
generated by STARGAZE).  A galaxy does not directly use the values
of its parameters, but instead passes the values on to the stars
and galaxies of which it is composed.  Simply by changing the parameter
values, many copies of a single star or galaxy can be used
in very different ways.  For example, a ten-pole IIR filter can
have its tap values defined in terms of parameters, so that
the same filter can be used in many applications.
.lp
BLOSIM maintains a data structure internally
called the \fIParameter Stack\fR.
Parameters are set by placing the values on the
Parameter Stack, and then copying them to the
appropriate star or galaxy.
After the values are copied, the Parameter Stack is cleared.
The copying is done
automatically when a star or galaxy is created,
so usually you specify what the parameters are
\fIbefore\fR you create the star or galaxy.
.lp
The \fBparam\fR command places a parameter value
of the specified type on the
Parameter Stack. The command must take one of the
following forms: 
.(b
Blosim> \fBparam int \fIinteger\fR
.)b
The parameter is an integer value.
.(b
Blosim> \fBparam float \fIfloating-point\fR
.)b
The parameter is a floating-point value.
.(b
Blosim> \fBparam array \fIsize value1 value2 ...\fR
.)b
The parameter is an array of values.  The first number gives the
size of the array and must be an integer.  The rest of the
numbers list the contents of the array, which may be floating-point.
\fBAn array may not contain more than ten elements.\fR
.(b
Blosim> \fBparam file \fIfile name\fR
.)b
The parameter is the name of a file, which may be
any UNIX file name.
.(b
Blosim> \fBparam function \fIfunction\fB.c\fR
.)b
The parameter is the name of a function.  The function
name must have a \*(lq.c\*(rq appended to it, indicating that
the function will be in found in the file \fIfunction.c\fR.
.(b
Blosim> \fBparam default\fR
.)b
The star is to use a default value for this parameter.
\fBThis default value must be set within the star program.\fR
.(b
Blosim> \fBparam arg \fIn\fR
.)b
The parameter value is to be obtained from the \fInth\fR parameter
of the block's parent galaxy.  The value is copied just before
the simulation is run.
.lp
To display the contents of the Parameter Stack, type
.(b
Blosim> \fBdisplay p\fR
.)b
.lp
The \fBunparam\fR command removes the most recently set parameter
value from the Parameter Stack:
.(b
Blosim> \fBunparam\fR
.)b
.lp
To change the parameters of a star or galaxy that has already
been created, use the \fBchange p\fR command:
.(b
Blosim> \fBchange p\fR
.)b
This will copy the contents of the Parameter Stack into the
parameters of the Current Block, and clear the Parameter
Stack.
.sh 1 "Connecting and Disconnecting"
.lp
To connect two blocks (stars or galaxies) together, use the
\fBconnect\fR command:
.(l
Blosim> \fBconnect\fR \fIstar1 outnumber star2 innumber\fR
.)l
This causes a data connection to be created, allowing data samples to
move from the \fIoutnumber\fR output of \fIstar1\fR to the
\fIinnumber\fR input of \fIstar2\fR.  Data samples can \fIonly\fR
move from \fIstar1\fR to \fIstar2\fR,
not in the reverse direction.
.lp
To connect a block to the output of the block's parent galaxy,
use this form of the \fBconnect\fR command:
.(l
Blosim> \fBconnect\fR \fIstar1 outnumber\fR \fBoutput\fR \fIgaloutnumber\fR
.)l
This will direct data samples from the \fIoutnumber\fR output
of \fIstar1\fR
out through the \fIgaloutnumber\fR output of \fIstar1\fR's parent
galaxy. 
.lp
Similarly,
.(l
Blosim> \fBconnect\fR \fBinput\fR \fIgalinnumber star2 innumber\fR
.)l
causes data samples that arrive at this galaxy's \fIgalinnumber\fR input
to be directed to the \fIinnumber\fR input to \fIstar2\fR.
.lp
\fBDisconnect\fR has an identical syntax,
and is used to remove a data link
between two blocks:
.(b
Blosim> \fBdisconnect\fR \fIstar1 outnumber star2 innumber\fR
Blosim> \fBdisconnect\fR \fIstar1 outnumber\fR \fBoutput\fR \fIgaloutnumber\fR
Blosim> \fBdisconnect\fR \fBinput\fR \fIgalinnumber star2 innumber\fR
.)b
It is an error to connect or disconnect from/to blocks that do not
exist.
.sh 1 "Running a Simulation"
.lp
To run a simulation, use the \fBrun\fR command:
.(l
Blosim> \fBrun\fR
.)l
A simulation will not run if any one of the following conditions 
is present:
.np
The universe is empty.
.np
The universe contains empty galaxies.
.np
A block within the universe is connected to an input or output.
.lp
In addition, a \fIstar error code\fR will be generated if one
of the following conditions is true:
.np
A star has an incorrect number of input or output connections.
.np
A star has an incorrect number of parameters,
or one of its parameters is of the wrong type.
.np 
A star uses an \fIarg\fR parameter that has not been 
properly set in the star's parent galaxy.
.lp
If there are no errors, the simulation will halt when no star has
data samples left on any of its input buffers.
\fBIMPORTANT:\fR
Any stars that are sources of data samples
must be designed to limit the number of samples they output,
otherwise the simulation will continue indefinitely.
.sh 1 "The Current Block"
.lp
The current galaxy always has a \fICurrent Block\fR associated with it.
(If the current galaxy is empty, the Current Block is a NULL.)
The Current Block is used by several other BLOSIM commands.
When \fBstar\fR or \fBgalaxy\fR is used to create a new block,
that block becomes the new Current Block.
.lp
The Current Block is displayed after several commands,
particularly those involved with movement through the topology.
See movement commands below.
The Current Block is displayed in one of the following formats:
.(b
\ Current Block: star\fI starpath/starname starfile\fR
\ Current Block: star\fI starpath/starname libstarname\fR (lib)
\ Current Block: galaxy\fI galaxypath/galaxyname galaxyfile\fR
.)b
The starpath and galaxypath show the complete hierarchy up to the
top (universe) level.
.sh 1 "Moving About"
.lp
To change the Current Block, use the \fBforward\fR,
\fBback\fR, and \fBto\fR commands:
.(b
Blosim> \fBforward\fR
Blosim> \fBback\fR
Blosim> \fBto\fR \fIname\fR
Blosim> \fBto\fR \fI(partial name)\fR
.)b
BLOSIM places the blocks that are in the current parent galaxy on
a list according to the order in which they were created.
You can view this ordering by using
the \fBdisplay a\fR command (see below).
The \fBforward\fR command will cause
the block that is listed after the existing Current Block 
to become the new Current Block.
Likewise, the \fBback\fR command will cause the block
that is listed before the existing Current Block
to become the new Current Block.
The \fBto\fR command searches in a forward direction until it
finds a block with a name that matches the name or partial name 
you provide.  If the search reaches the end of the list, it starts
again at the beginning until it reaches the Current Block.
A message is printed if no match is found for the \fIname\fR pattern.
.lp 
To move through the block hierarchy, use the \fBup\fR and
\fBdown\fR commands:
.(b
Blosim> \fBup\fR
Blosim> \fBdown\fR
.)b
The \fBup\fR command causes BLOSIM to move the current galaxy
up one level.
The previous galaxy block becomes the new Current Block.
If the current parent galaxy is the universe,
\fBup\fR will print an error message.
Similarly, the \fBdown\fR command causes the Current Block
to become the new parent galaxy.
This command is in error if the Current Block is a star.
.sh 1 "Display and Info"
.lp
The \fBdisplay\fR command is used to print information
about the current state of the BLOSIM environment.
We have already seen the \fBdisplay on\fR and \fBdisplay off\fR
commands, which control the display of the Current Block, and
the \fBdisplay p\fR command, which displays the contents of
the Parameter Stack.
.lp
To display the complete topology of the current parent
galaxy, type:
.(b
Blosim> \fBdisplay a\fR
.)b
This will print the name of each star and galaxy, what
values their parameters are set to, and 
how they are interconnected.  This information
is displayed in terms of the commands that would
have to be given to BLOSIM to generate the existing universe.
Thus, a star named \*(lqmystar0\*(rq with an integer
parameter set to 23 would be displayed as follows:
.(b
\ param int 23
\ star mystar0 mystar.o
.)b
.lp
To see what stars are in the standard star library, use the 
\fBdisplay l\fR command:
.(b
Blosim> \fBdisplay l\fR
.)b
This will print out the module name of each library star.
.lp
To see what external star object code has already been loaded
into memory, use the \fBdisplay s\fR command:
.(b
Blosim> \fBdisplay s\fR
.)b
The names of star object code files that are resident
in memory are displayed.
.lp
The \fBdisplay g\fR command shows which galaxy topology
files BLOSIM knows about:
.(b
Blosim> \fBdisplay g\fR
.)b
These are topology files that have previously been used
in a \fBgalaxy\fR or \fBdefine\fR command, or have been
loaded by BLOSIM.
.lp
To display the name of the Current Block while
the automatic display is turned off, use \fBdisplay\fR
without an argument:
.(b
Blosim> \fBdisplay\fR
.)b
The \fBinfo\fR command gives more complete information 
about the Current Block. It lists what values its parameters
have, and what its connections are:
.(b
Blosim> \fBinfo\fR
.)b
.sh 1 "Loading and Storing"
.lp
The load command causes BLOSIM to try to load a topology from
a file:
.(b
Blosim> \fBload\fR
Blosim> \fBload \fIfile\fR
.)b
If no file is specified, and the currently
displayed galaxy is the universe, 
BLOSIM loads the default universe file
(\fBuniverse.t\fR, if no other is specified with the -t option).
If the currently displayed galaxy is not the universe,
BLOSIM tries to load the file associated with this galaxy.
.lp
The \fBload\fR command acts recursively; that is,
if any galaxies are part of the loaded topology, their
topologies are also loaded.  During the load, if any
topology files are in error, the name of the offending
file and the line number are printed along with the
appropriate error message. Galaxies that could not be loaded,
or that contain galaxies that could not be loaded, are
given the designation \*(lqbad\*(rq.  A universe containing
bad galaxies  can not be run (see the \fBgalaxy\fR command
above).
.lp
To store a galaxy, use the \fBstore\fR command:
.(b
Blosim> \fBstore\fR
Blosim> \fBstore \fIfile\fR
.)b
If no file is specified, the rules are the same as for
\fBload\fR. Note that \fBstore\fR is different from load
in that it \fIonly\fR stores the 
topology of the currently displayed galaxy, not the
topology of any of its children.
.sh 1 "Removing Blocks"
.lp 
The \fBremove\fR command first disconnects the Current Block from
all blocks to which it is connected.  The Current Block is then
removed.  If the parent galaxy is not empty, another block becomes
the Current Block.
.(l
Blosim> \fBremove\fR
.)l
.lp
The \fBnew\fR commands removes every block in the parent galaxy:
.(l
Blosim> \fBnew\fR
.)l
\fBWarning: This command does not prompt you for a confirmation.\fR
.sh 1 "Search Paths"
.lp
By default, BLOSIM only searches
in the current working directory
for star and galaxy files.
You can use the \fBpath\fR to specify alternate paths to search in.
This allows you to keep all your stars in a
set of directories, and galaxies
in a different set.
.(b
Blosim> \fBpath s\fR \fIpathname\fR
.)b
adds \fIpathname\fR to the list of directories that are searched
when BLOSIM looks for a star file, and
.(b
Blosim> \fBpath g\fR \fIpathname\fR
.)b
adds \fIpathname\fR to the list of directories that are searched
for galaxies.  You can use \*(lq.\*(rq or \*(lq..\*(rq as part
of the path.
.lp
\fBNote: \fBpath\fR does \fBnot\fR expand tildes (~).
.sh 1 "Commands to the Shell"
.lp 
To execute a shell command, use the \fBsh\fR command:
.(b
Blosim> \fBsh\fR \fIcommand\fR
.)b
The \fBsh\fR command always uses the Bourne Shell, never the C-Shell.
.sh 1 "Aliases"
.lp
BLOSIM contains an alias facility.  When a command is presented
to BLOSIM, each word that makes up the command is checked to see
if it is an alias for a string.  If so, the string is
substituted for the alias before the command is executed.
For example, if \*(lqpi\*(rq is an alias for \*(lqparam int\*(rq,
then
.(b
Blosim> pi 20
.)b
is equivalent to 
.(b
Blosim> param int 20
.)b
.lp
To set an alias, use the \fBalias\fR command:
.(b
Blosim> \fBalias\fR \fIalias\fR \fIsubstitution\fR
.)b
To remove the substitution, use \fBunalias\fR:
.(b
Blosim> \fBunalias\fR \fIalias\fR
.)b
The alias expansion is disabled when setting or unsetting
aliases, in order to avoid some disturbing situations.
.lp
To display what aliases are currently active, type \fBalias\fR
without an argument:
.(b
Blosim> \fBalias\fR
.)b
.uh "References"
.sp 2
.in 5
.ti 0
[1] D. J. Hait and D. G. Messerschmitt, \fIA BLOSIM Tutorial\fR
.br
.sp
.ti 0
[2] D. G. Messerschmitt, \fISTARGAZE User's Manual\fR (in print)
.bp
.uh "Appendix A"
.sp
.(c
\fBAdding Your Own Standard Library\fR
.)c
(modifications: ljfaber 5/88)
.lp
This section describes how to recompile BLOSIM to include your
own set of star programs as the standard library.  First, in the
files that were created from the BLOSIM distribution tape, find
the directory /SRC.  In this directory are the source files
for BLOSIM.  Locate the source file \fBstar_lib.c\fR.
This file contains a list of the modules to be included
in the standard library.
.lp
First make a copy of \fBstar_lib.c\fR,
just in case you make a mistake (call it \fBstar_lib.save\fR).
Now using the editor, look for the following lines in \fBstar_lib.c\fR:
.(b
/* Function declarations */

extern	add(),
	delay(),
	fork(),
	impulse(),
	multiply(),
	print_file(),
	read_file(),
	time()
;
.)b

In the same format, change these lines so that they reflect the
names of the modules you wish to include in your own library.
For example, if you wished to include programming from the
file \*(lqfilter.s\*(rq, you would make sure that  \*(lqfilter()\(rq
was included among the \fIFunction Definitions\fR.
.lp
Now look for the following lines:
.(b
/* gal_table Definition */

ENTRY gal_table[MAX_STAR_FILES] =  {

/* function		name			ident  */

{ NULL,		"terminal",		"@@@"	},
{ fork,			"fork",		"@@@"	},
{ delay,		"delay",		"@@@"	},
{ add,			"add",			"@@@"	},
{ multiply,		"multiply",		"MUL"	},
{ impulse,		"impulse",		"IMP"		},
{ print_file,		"printfile",		"PRF"	},
{ read_file,		"readfile",		"RDF"	},
{ time,		"time",		"@@@"	}

}
;

.)b
Again, following the same format, add an initiallization line
that gives the name of the function and the name of the module.
For example, our filter would be entered as
.(b
{ filter,		"filter",		"@@@"	},
.)b
For the \fIident\fR field, use \*(lq@@@\*(rq.  \fBImportant:
No matter what modules you include in your own library,
be sure to include the following line as the first entry:\fR
.(b
{ NULL,		"terminal",		"@@@"	},
.)b
BLOSIM uses this module internally.
.lp
Now adjust the \fBgal_count\fR line:
.(b
int gal_count = 9;
.)b
This variable should be set to one more than the number of 
modules in the library (or, the number of modules plus one
for the NULL module).
Now save this file and exit the editor.
.lp
Next, create an object (.o) file for the new star
in the /STARS directory.
You can use the \fBsto\fR command to create a .o file from a .s file.
From the /SRC directory, execute the \fBmklibstar\fR command to update
the archive file named \fBlibstar.a\fR
which contains the object code for all the library modules.
Now, type \fBmake\fR.  You should see the system recompile \fBstar_lib.c\fR
(and maybe some other files as well), and then link everything together.
.lp
Once this has been completed, invoke BLOSIM and give the
\fBdisplay l\fR command.  BLOSIM should list the modules in
your new standard library.
.bp
.uh "Appendix B"
.sp
.(c
\fBBLOSIM Command Summary\fR
.)c
.ps 12
.vs 14p
.in 1i
.ti 0
\fBalias\fR
.br
display all aliases
.ti 0
\fBalias \fIalias string\fR
.br
alias the word \fIalias\fR to \fIstring\fR
.ti 0
\fBback\fR
.br
move the Current Block back one
.ti 0
\fBchange p\fR
.br
change the parameters of the Current Block
.ti 0
\fBconnect\fI block1 num1 block2 num2\fR
.br
connect output number \fInum1\fR of \fIblock1\fR
to input number \fInum2\fR of \fIblock2\fR
.ti 0
\fBconnect\fI block1 num1 \fBoutput\fI num2\fR
.br
connect output number \fInum1\fR of \fIblock1\fR 
to input number \fInum2\fR of the parent galaxy
.ti 0
\fBconnect input \fInum1 block2 num2\fR
.br
connect input number \fInum1\fR of
the parent galaxy to input number
\fInum2\fR of \fIblock2\fR
.ti 0
\fBdefine s \fIstarfile\fR
.br
load the star programming in
\fIstarfile\fR into memory
.ti 0
\fBdisconnect\fI block1 num1 block2 num2\fR
.br
disconnect output number \fInum1\fR of
\fIblock1\fR from input number
\fInum2\fR of \fIblock2\fR
.ti 0
\fBdisconnect\fI block1 num1 \fBoutput\fI num2\fR
.br
disconnect output number \fInum1\fR of
\fIblock1\fR from input number 
\fInum2\fR of the parent galaxy
.ti 0
\fBdisconnect input \fInum1 block2 num2\fR
.br 0
disconnect input number \fInum1\fR of
the parent galaxy from input number
\fInum2\fR of \fIblock2\fR
.ti 0
\fBdisplay a\fR
.br
display the entire universe
.ti 0
\fBdisplay l\fR
.br 
display stars in the library
.ti 0
\fBdisplay off\fR
.br
don't display the Current Block
after each command
.ti 0
\fBdisplay on\fR
.br
display the Current Block
after each command
.ti 0
\fBdisplay p\fR
.br
display the Parameter Stack
.ti 0
\fBdisplay s\fR
.br
display the current star list
.ti 0
\fBdisplay\fR
.br
display the Current Block
.ti 0
\fBdown\fR
.br
move down the hierarchy
.ti 0
\fBexit\fR
.br
leave BLOSIM
.ti 0
\fBforward\fR
.br
move the Current Block forward one
.ti 0
\fBgalaxy \fIname topfile\fR
.br
create a galaxy named \fIname\fR with
topology file \fItopfile\fR
.ti 0
\fBinfo\fR
.br
display information about
the Current Block
.ti 0
\fBload\fR
.br
load a topology from the parent
galaxy's topology file
.ti 0
\fBload \fIfile\fR
.br
load a topology from the specified
topology file
.ti 0
\fBnew\fR
.br
destroy the universe
.ti 0
\fBparam\fI type value\fR
.br
place a parameter value on 
the Parameter Stack
.ti 0
\fBpath s \fIpathname\fR
.br
add a directory to the star
path search list
.ti 0
\fBpath g \fIpathname\fR
.br
add a directory to the galaxy
path search list
.ti 0
\fBquit\fR
.br
leave BLOSIM
.ti 0
\fBremove\fR
.br
remove the Current Block
.ti 0
\fBrun\fR
.br
run a simulation
.ti 0
\fBsh \fIcommand\fR
.br
send a command to the Shell
.ti 0
\fBstar \fIname module\fR
.br
create a library star from the specified
library object module
.ti 0
\fBstar \fIname file\fR
.br
create a star from the specified file
.ti 0
\fBstore\fR
.br
store a topology
.ti 0
\fBto \fIname\fR
.br
make \fIname\fR the Current Block
.ti 0
\fBunalias\fI alias\fR
.br
remove an alias
.ti 0
\fBunparam\fR
.br
remove a parameter from 
the Parameter Stack
.ti 0
\fBup\fR
.br
move up one level in block hierarchy
