.po +0.5i
.fo ''%''
.(l C
.sz +2
.b "STARGAZE -- A Preprocessor for BLOSIM User STAR Functions"
.sz
.sp2
David G. Messerschmitt
Department of Electrical Engineering and Computer Science
University of California
Berkeley, California 94720
.sp2
\*(td
.)l
.sh 1 Introduction
.pp
User STAR functions for BLOSIM,
in addition to implementing the functionality of a simulation,
must do a number of things to effect the interface to BLOSIM,
including accepting arguments, allocating storage for state variables,
checking the size of parameter storage and the number of input and output buffers,
defaulting parameter values and initializing state variables, etc.
STARGAZE is a program which facilitates and automates many of these
functions, thereby permitting the user to concentrate on the
simulation functionality rather than the interface to BLOSIM.
.pp
STARGAZE is a preprocessor for user STAR functions which allows
the user to express many of the interface parameters in a form
similar to and compatible with the topology specification,
and then turns this specification into C code for compilation
and execution.
.sh 1 "STARGAZE User Specifications"
.pp
STARGAZE provides a number of control line formats, which are
described in this section.
.sh 2 "Input and Output Files"
.pp
The file provided to STARGAZE must have a ".s" extender;
that is, be of the form "name.s".
STARGAZE generates a C program in a file "name.c"
which contains the C function name() which is the corresponding
user STAR function.
Hence, the name of the file and the name of the C function are
automatically consistent as required by FILETOP.
.pp
The STARGAZE program is automatically invoked as a part of the
user interface.
Whenever the user loads a new star into the simulation,
STARGAZE is run if any changes have been made to the ".s" file
since the last execution of STARGAZE.
The user can also run STARGAZE directly if desired.
.sh 2 "#Include and #Define"
.pp
Any #include or #define lines which are included in the input file
are added to the output file automatically.
The files <stdio.h>, "param.h", "star.h", and "type.h" are automatically
included by STARGAZE, and hence do not need to be included by
the user.
.sh 2 "The C Program"
.pp
The user must of course provide in the input file the C program
which implements the functionality of the user STAR function.
This C program generally includes three parts:
.ip 1.
Declarations of variables.
.ip 2.
Initialization code which is to be executed the first time the
user STAR routine is called by BLOSIM.
.ip 3.
The main body of code which is executed every time the user STAR
function is called.
This code accesses the buffers and processes the samples.
.lp
Since STARGAZE automatically generates the initialization code
for checking the number of buffers,
checking the number of parameters and their types,
defaulting the parameters,
and allocating storage for state variables and initializing them,
many user STAR routines will not require any additional
initialization code.
.pp
STARGAZE recognizes these three types of code because they are
imbedded between control lines.
In particular, declarations of variables are imbedded between
the lines
.(l
.b declarations
....
....
....
.b end
.)l
and the initialization code is imbedded between the lines
.(l
.b initialization_code
....
....
....
.b end
.)l
and finally the main code is imbedded between the lines
.(l
.b main_code
....
....
....
.b end
.)l
.lp
Of course, none of the STARGAZE control lines should be
imbedded between these control lines,
as STARGAZE simply copies indiscriminately everything between these control
lines to the C program that it generates.
.sh 3 "Available Functions in User Code"
.pp
The following functions are available for use in the user-written C code
which is included in the STARGAZE input file.
.ip
.i avail(buffer_no)
returns the number of samples available to be read
on buffer
.i buffer_no.
.ip
.i min_avail()
returns the minimum number of samples available on all the input buffers.
.ip
.i it_in(buffer_no)
increments time on the input buffer number
.i buffer_no
and returns the number of samples that were available in the buffer
before the increment.
If it was not possible to increment time (no more samples available),
then
.i it_in(buffer_no)
does nothing and returns 0.
.ip
.i it_out(buffer_no)
increments time on output buffer number
.i buffer_no
and returns nothing in particular.
Specifically, it allocates a new sample on the output buffer,
which becomes the sample with delay 0.
There is no limit imposed on the size of a buffer,
so it is unnecessary to check if a buffer is full before calling
.i it_out().
.ip
.i no_input_buffers()
and
.i no_output_buffers()
return respectively the number of input and output buffers connected
to the STAR by the topology definition.
.pp
Normally the user refers to input and output buffers by name, if they
are floating point numbers.
Thus, if "sample" is the name of an output buffer,
then the last output sample would be referred to as
.i sample(0)
and the output sample
.i delay
in the past would be referred to as
.i sample(delay).
A new output sample would be generated by calling
.i it_out(buffer_no)
and then referring to the new sample being generated as
.i sample(0).
Similarly, the output sample
.i delay
samples in the past would be referred to as
.i sample(delay).
.pp
However, in the case of buffers which do not store floating values,
or where there are a variable number of buffers,
the following routines are available to manipulate buffers.
.ip
.i it_out(buffer_no)
adds a new sample to the output buffer number
.i buffer_no
(as mentioned previously)
and returns a pointer to that sample (we were just kidding that
it returns nothing in particular before!).
For example, to set that new output sample for buffer_no 0
to an integer value 12,
we would say
.(l
	*(int *)it_out(0) = 12;
.)l
.ip
.i pout(buffer_no,delay)
returns a pointer to the sample
.i delay
in the past for output buffer number
.i buffer_no.
.ip
.i pin(buffer_no,delay)
returns a pointer to the sample
.i delay
in the past for the input buffer number
.i buffer_no.
.ip
.i outf(buffer_no,delay)
and
.i inf(buffer_no,delay)
return not a pointer to a sample, but rather actual floating point
values. They of course assume the buffer is passing floating point
numbers.
.ip
.i set_dmin_in(buffer_no,delay_min)
is a routine which sets the minimum delay relative to the current input
sample for the input buffer number
.i buffer_no
to the value
.i delay_min,
an integer constant or parameter name.
The value of this minimum delay defaults to zero, so if the STAR will
access the current sample it is unnecessary to call this function.
.ip
.i set_dmax_in(buffer_no,delay_max)
and
.i set_dmax_out(buffer_no,delay_max)
set respectively the maximum delay relative to the current sample of
an input buffer and the maximum delay relative to the latest output
sample for an output buffer with which the buffer is accessed
to
.i delay_max,
which is an integer constant or parameter name.
The maximum delay, in the case of an input buffer, defaults to be the
same as the minimum delay.
In the case of an output buffer, it defaults to zero.
.ip
.i set_cellsize_in(buffer_no,size)
and
.i set_cellsize_out(buffer_no,size)
are routines which set the size of one cell in the buffer to a fixed
integer
.i size.
This value defaults to the appropriate size for a floating value,
and does not need to be set for that case. But for example if the
buffer transmits integer values, then
.i size
would be replaced by
.i sizeof(int).
.sh 2 "Specifying Input and Output Buffer Names"
.lp
.b input_buffers
.sp
.ip
.b delay_min
=
.i delay_value;
.ip
.b delay_max
=
.i delay_value;
.ip
.i "type buffer_name;"
.lp
.b end
.lp
.b output_buffers
.ip
.b delay_max
=
.i delay_value;
.ip
.i "type buffer_name;"
.lp
.b end
.lp
In these statements,
.ip
.i delay_value
is either an integer constant, or an integer variable
which is an input parameter,
.ip
.i type
is the type of variable, either
.b float
or
.b int,
and
.ip
.i buffer_name
is the name of the buffer as referred to in the
main program.
.lp
Note that the delay statements are unneeded if only current buffer
samples are referenced.
(See previous section descriptions of
.i set_dmin_in,
etc.)
If used, these statements must be placed just prior to their respective
buffer name declaration.
.sh 2 "Passing Parameters"
.pp
The only parameters which can be passed
are integers, floating point numbers, and strings.
.pp
Parameters are passed to the STAR by including their names and types
and default values in control lines between the two lines
.lp
.b parameters
.sp
.ip
.i "type parameter_name [ = default_value ];"
.sp
.lp
.b end
.)l
where
.ip
.i type
is the identification of the type of parameter.
There are presently five possibilities for
.i type,
which are specifically
.b "int, float, file, function"
and
.b array.
These possibilities are consistent with FILETOP.
.ip
.i parameter_name
is the identifier of the parameter as it is referred to in 
the initialization and main code.
.ip
.i default_value
is a constant which is the value of the parameter
if it is not set by the user in the topology description.
If the topology specifies a parameter as default
which does not have a
.i default_value
specified within the star program,
an error will result and the simulation will be terminated.
Note that the \fBarray\fR parameter type
does not support default values.
If you use a parameter array, the size and initial values
must be set in the topology file.
(This is enforced by the blosim program.)
.lp
Examples of
.b parameters....end
lines would include
.ip
.b int
no_samples;
.ip
.b float
time_scale = 1.0;
.ip
.b array
myarray;
.ip
.b file
file_name = "stdin";
.ip
.b function
impulse_response = "hfilt";
.lp
In the first case the parameter is not given a default value (in this
example) and hence must be specified in the topology routine.
In the third case with arrays, note that no "brackets" or dimensioning
should be used, and no default values can be given.
This must be done in the topology file.
The array values can be changed freely within the star program, however.
Also, an integer variable containing the size of the array will be
\fBautomatically\fR declared for use within the star program.
Its name will be assigned as the array name with prefix "n_";
thus in the example above, an integer "n_myarray" will be declared and
initialized with the size of "myarray".
.pp
In the fourth and fifth cases, the parameter is a string,
and defaults to "stdin" and "hfilt" respectively.
STARGAZE does not provide automatic conversion of these
strings to pointers to a file or function.
This must be done by the user between the lines
.b initialization_code
and
.b end.
Here is an example to illustrate conversion of a parameter of type
.b file
to a file pointer and its use to print out to that file:
.hl
.(l
parameters

	file file_name = "stdin";

end

declarations

	FILE *fp,*fopen();

end

initialization_code

	if(strcmp(file_name,"stdin") != 0) {
		if(fp = fopen(file_name,"r")) == NULL) return(1);
		}
	else fp = stdin;
end

main_code

	fscanf(fp,.....);

end
.)l
.hl
The result is that if the parameter specifies a file which does
not exist (cannot be opened to read),
then the STAR will return a non-zero error code, terminating
the simulation.
If the parameter defaults,
the input will come from standard input
(recall that stdin is a file pointer for standard input).
.pp
Here is an example initialization and use for the
function parameter:
.hl
.(l
parameters

	function impulse_response = "hfilt";

end

declarations

	/* PFI is a typedef for function returning an integer
	   defined in "type.h" */
	PFI function,funct_list();
	int hfilt();
end

initialization_code
	if(strcmp(impulse_response,"hfilt") != 0) {
		if((function = funct_list(impulse_response)) == NULL)
			return(1);
		}
	else function = hfilt;
end

main_code
	/* call the function returning the impulse response */
	... = (*function)(...);
end
.)l
.hl
.lp
In this example, the default function hfilt() is declared
right inside the STAR routine to be a function pointer,
but any other function name passed as a parameter is converted
to a function pointer using the look-up table funct_list()
described in FILETOP [2].
.sh 2 "State Variables"
.pp
A state variable can be defined by
.lp
.b states
.ip
.i "type state_var_name [ = init_value];"
.lp
.b end
.lp
where
.ip
.i state_var_name
is an identifier for the state variable
which will be used in the program,
and
.ip
.i type
is the type of the variable
(float, int, FILE*, float*, etc.)
which must not contain any blanks
(note the absence of blanks between FILE and * for instance), and
.ip
.i init_value
is the initial value the state variable is to assume, which
can be either a constant or the name of a parameter.
.lp
Examples of state variable definitions would be
.(l
states

	int no_samples = 100;
	float delay;
	float array[25];
	FILE* fp;
	char[] name_star = "harvey";

end
.)l
.lp
These examples illustrate that the type of each state variable
need not be
.b int
,
.b float
,
or
.b char
,
but rather can be more complex types such as pointers to
objects of this type.
.sh 2 "Template STAR"
.pp
You should have a template STAR "template.s"
which has all the control structures and comments contained in <>
that direct you as to what needs to be added.
Editing this STAR, deleting the comments and adding your own
customized code, is an easy way to develop a new STAR definition.
An alternate approach is to edit another similar preexisting STAR
definition.
It is almost never appropriate to start from scratch!
.sh 1 Example
.pp
As an example of a complete STAR function,
let us take delay().
Here is a STARGAZE input file for delay(), by convention
stored in a file delay.s:
.hl
.(l L
.so /CCSP/blosim/B3DIST/STARS/delay.s
.)l
.hl
.lp
and here is the corresponding C program stored in a file
delay.c by STARGAZE:
.hl
.(l L
.so /CCSP/blosim/B3DIST/STARS/delay.c
.)l
.hl
