.po +0.5i
.nr pp 12
.nr sp 12
.nr tp 12
.nr qp 12
.nr fp 10
.fo ''%''
.(b C
.ps 14
.b "A BLOSIM Tutorial"
.ps 12
.sp 2
David J. Hait
David G. Messerschmitt
Department of Electrical Engineering and Computer Science
University of California
Berkeley, California 94720
.sp 2
\*(td
.)b
.he 'A BLOSIM Tutorial''\*(td'
.sp 2
.(f
This work was supported by the National Science Foundation,
under Grant ECS-82-11071.
.)f
.uh Introduction
.pp
This tutorial introduces you BLOSIM,
a signal processing simulation program.
BLOSIM can be used design simulations for a large number of signal 
processing and communications applications, with a minimum of 
programming overhead.  The sections below will guide you through
the use of BLOSIM to create a few very simple simulations. 
.pp
In this tutorial,
whenever you are asked to type in text at
your terminal or workstation,
the text is shown in
\fBbold-faced type\fR.
This is to prevent you from confusing it with BLOSIM's output messages, which
are shown in a normal font.
Naturally, when you use BLOSIM, everything
will be displayed on your terminal in the same type style.
.uh "Invoking BLOSIM"
.pp 
Before you start, you should make sure that BLOSIM has been installed in an
appropriate directory on your system (probably /usr/local, though this is 
different from installation to installation). 
To invoke BLOSIM, type
.(l F
% \fBblosim\fR
.)l
from the Shell.  The system will respond
.(l F
\ Blosim 3.0  Topology file: universe.t
.)l
followed by the prompt
.(l F
Blosim>
.)l
BLOSIM is now ready to accept commands.
.uh "Stars, Galaxies, and the Universe"
.pp
In BLOSIM, you construct simulations out of a number of building blocks. 
These building blocks, known as \fIstars\fR,
correspond to small pieces of simulation programming.  Each
star has inputs and/or outputs over which data samples are passed.
By connecting these
stars together in a way analogous to creating a block diagram, you can
execute very large simulations.  Since stars can be used from
simulation to simulation, you can use BLOSIM to build up a library of stars
that is suited to your own application needs.
To help get you started, BLOSIM is
supplied with a small library of star object code that is automatically 
loaded into memory when BLOSIM is invoked.
.pp
You can also combine building blocks together
to form larger building blocks, which can in turn be used to 
create more advanced simulations.  Building blocks that are
created out of smaller ones in this way are known as  \fIgalaxies\fR.
These concepts will become clearer as we go on.
.pp
The collection of stars and
galaxies in a simulation is collectively known as
the \fIuniverse\fR. When you first
invoke BLOSIM, the universe is empty.
To create a BLOSIM simulation, you must first bring
copies of the stars and
galaxies that you are going to use into the universe.
You can then connect these stars and
galaxies together; that is, you specify the way data samples are to flow
between them. The names of the stars and galaxies, and the specification
of how they are interconnected, is known as a \fItopology\fR. 
A topology can be stored on a file, so that the arrangement of the
universe can be saved for later simulations. Once you have
created a universe, either by creating it with BLOSIM commands or loading
its topology from a file, you can run the simulation to get the desired
results.
.uh "The Standard Star Library"
.pp
To get started, the first thing you should do is see
what stars can be created out of BLOSIM's 
standard library. To do this, type
.(l F
Blosim> \fBdisplay l\fR
.)l
BLOSIM will display a list of \fImodules\fR were loaded as
part of its standard library.  A star can be created by
specifying the appropriate module. Find the modules
\fBimpulse\fR and \fBprintfile\fR
in this list. You should realize that even though these
program segments are in the standard library, there is nothing special 
about them. They are identical in format to the code you will be
writing yourself later on.
.uh "Parameters"
.pp
Before you actually use one of these stars, let's talk a bit about
\fIparameters\fR.
A parameter is a value that you specify to affect
the way the star works.
For example, the library module \fBimpulse\fR
is used to generate an impulse data sequence, that is, a 1 sample followed
by a number of 0 samples.  This number of trailing 0's is not fixed 
anywhere in the program for \fBimpulse\fR,
but is specified by a parameter to an \fBimpulse\fR star.  
In fact, you can create several stars based on \fBimpulse\fR, with each 
star having a different parameter value.  In this manner, 
simulation programming
can be written in a very general manner, with parameters not set
until the time they are used.
.pp
Galaxies can also take parameters.  In this case, the galaxy holds
the parameter values so that the stars which make up the galaxy can
use them.  A special parameter type called \fIarg\fR is used
by the star to tell the galaxy which parameter the star needs.  
For example, if our \fBimpulse\fR star above has a parameter of
\fBarg 3\fR, it means that the number of 0 samples is equal to the
value of the third parameter to the galaxy in which \fBimpulse\fR is
contained.  Just before the simulation is executed, the value of this
third parameter will be copied to \fBimpulse\fR.
.pp
The programming for the star specifies how many 
parameters the star expects there to be,
what their types are (floating-point, integer, etc), and how they are to
be used.  \fBImpulse\fR stars
take only one parameter of type integer, and use this
parameter to send out the proper number of 0 samples. Any other number or
type of parameters will cause an \fBimpulse\fR star
to complain.
.pp
.uh "Using Stars"
.pp
Now you are ready to create a star.  First tell BLOSIM that the next 
star (or galaxy) that we are going to create will need an integer parameter:
.(l F
Blosim>  \fBparam int  10\fR
.)l
This places the value 10 on the \fIParameter Stack\fR,
a scratchpad list of parameters that BLOSIM keeps around until
the star (or galaxy) is actually created.  When using BLOSIM,
you must first declare what parameter values your
star will need, and then create the star.
To see what is currently on the Parameter Stack, type
.(l F
Blosim> \fBdisplay p\fR
.)l
You should see
.(l F
param int 10
.)l
indicating that the Parameter Stack currently contains one value (integer 10).
If you make a mistake, you can remove the most recent value from the parameter
stack by typing
.(l F
Blosim> \fBunparam\fR
.)l
Once you have the correct parameter values on the stack,
create a copy of the \fBimpulse\fR
star, by typing
.(l F
Blosim> \fBstar impulse0 impulse\fR
.)l
\fBImpulse0\fR is the name of the star
you are creating, while \fBimpulse\fR
is the library module which contains its programming.
Stars may have a name
composed of any alphanumeric string of reasonable length, as
long as no other stars or galaxies in this universe have
the same name. The parameters that were on the
Parameter Stack have now been transferred to \fBimpulse0\fR.
You can verify this by typing the \fBdisplay p\fR
command again; BLOSIM prints nothing, indicating that the Parameter Stack is 
now empty.
.pp
Notice that now, after each command, BLOSIM prints the message 
.(l F
\ Current Block: star impulse0 impulse (lib)
.)l
BLOSIM keeps track of the most recently created block, and displays
it after each command.  The Current Block is used by some of the other
BLOSIM commands, as we shall see later.  To disable printing of the 
\ Current Block after each command, type
.(l F
Blosim> \fBdisplay off\fR
.)l
To re-enable it, type
.(l F
Blosim> \fBdisplay on\fR
.)l
.pp
In a likewise manner, create the star
\fBprintfile0\fR, based on the module \fBprintfile\fR,
by typing the sequence
.(l 
Blosim> \fBparam default\fR
\ Current Block: star impulse impulse0 (lib)
Blosim> \fBstar printfile0 printfile\fR
\ Current Block: star printfile0 printfile (lib)
.)l
Here, rather than specifying a value for the parameter of 
\fBprintfile\fR,
you have told \fBprintfile\fR
to use its internal default value for the first parameter. 
\fBPrintfile\fR is a star that takes data samples from its inputs and
places them in the file specified by the first parameter.  The default value
for this first parameter is the standard output.  Default parameter values
are specified by the star's programmer, and are included as a convenience
to the user.  You might just as well have substituted 
.(l F
Blosim> \fBparam file \*(lqout.out\*(rq\fR
.)l
for the \fBparam\fR
command above; if you had, the data samples being input to this star would
have been put into the file \*(lqout.out\*(rq.
.uh "Making Connections"
.pp
Now you have star that will generate data samples, and 
another star which will consume data samples. You need only to connect them
together to start running a simulation.  To connect an output of 
\fBimpulse0\fR to an input of \fBprintfile0\fR,
type
.(l F
Blosim> \fBconnect impulse0 0 printfile0 0\fR
.)l
In this command, you are telling BLOSIM
to connect the first output of
\fBimpulse0\fR (output number 0) to the first input of
\fBprintfile0\fR (input number 0).
Each star can have a large number of inputs and output (the exact number is
implementation dependent).
BLOSIM will allow you to connect any star's output to any star's input,
as long as both stars exist and the desired input or output is not 
already connected. It matters to the stars themselves, however, how
each input or output is connected. The
number of connections that a star has,
and how they are used, is specified by the star's programmer.  
\fBImpulse\fR (as well as \fBimpulse0\fR)
is designed to put output samples on one output connection. If you 
connect anything to its inputs, or connect its outputs to more than one
other star, you will get an error.  BLOSIM will not give you this error, 
however: it will come up only when you attempt run the simulation, as
a \fIuser error code\fR.
We will talk more about this later.
.uh "Checking the Universe"
.pp
Now you have a universe consisting of two stars connected together.
To see if BLOSIM's view of the universe agrees with your own, type
.(l
Blosim> \fBdisplay a\fR
.)l
BLOSIM will print every star or galaxy in the universe,
and all the connections between them.
The parameters for each star or galaxy are printed \fIbefore\fR
the star or galaxy itself; this corresponds exactly to the way that parameters
are first placed on the Parameter Stack, and then transferred to the next
star or galaxy created.
.uh "Running the Simulation"
.pp
Now you can see if your simulation will work correctly.  To run, type
.(l F
Blosim> \fBrun\fR
.)l
If you did everything right, the \fBimpulse0\fR
star should pass data samples on to the \fBprintfile0\fR
star, which then displays them on the standard output:
.(b
1.000000
0.000000
0.000000
0.000000
0.000000
0.000000
0.000000
0.000000
0.000000
0.000000
0.000000
.)b
Note that, as expected, there are ten 0 samples after the 1.
.uh "Changing the Parameters"
.pp
Suppose that once you have created a star or galaxy, you
wish to change its parameters.  You can do this with the 
\fBchange\fR command.  This command copies the values currently on the 
Parameter Stack into the Current Block. For an example, let us say that
you know want the \fBimpulse0\fR star to produce twenty 0's,
rather than ten, after the 1.  First, you must make
\fBimpulse0\fR the Current Block. 
The  \fBforward\fR, \fBback\fR and \fBto\fR commands allow you to move
around within the universe by changing the Current Block.
.(l
Blosim> \fBback\fR
\ Current Block: star impulse0 impulse (lib)
.)l
Now that \fBimpulse0\fR is the Current Block, we can go ahead and change its
parameter value. First put the desired value on the Parameter Stack:
.(l
Blosim> \fBparam int 20\fR
\ Current Block: star impulse0 impulse (lib)
.)l
If you want to, you can verify that the proper value was entered by using
the \fBdisplay p\fR command. Now type
.(l
Blosim> \fBchange p\fR
\ Current Block: star impulse0 impulse (lib)
.)l
The parameter is now changed.
.pp
You can verify the change you just made by typing \fBdisplay a\fR,
which shows the contents of the entire universe.  
If your universe is too big to display completely, you can use
the \fBinfo\fR command, which
tells you about the Current Block only:
.(b
Blosim> \fBinfo\fR
impulse0
Type: STAR
File: library

Parameters:
 0: int 20
Inputs:
 none
Outputs:
 0: printfile0 0
Current Block: star impulse0 impulse (lib)
.)b
Try to run again:
.(l
Blosim> \fBrun\fR
.)l
Now the impulse sequence should contain 20 zero samples.
.uh "Saving the Universe"
.pp
To save the universe that you have created, use the \fBstore\fR command:
.(l
Blosim> \fBstore\fR
.)l
Since you did not specify a file name, BLOSIM will store this universe
in the default topology file, which is 
\*(lquniverse.t\*(rq.  You will now be able
to run this simulation at a later time without having to rebuild the universe,
simply by loading \*(lquniverse.t\*(rq.
.uh "Creating a Galaxy"
.pp
Now that you have a familiarity with the basic commands
of BLOSIM, you are ready build more advanced simulations.
We mentioned before that stars can be combined to form galaxies, which can
then be connected in the same way that stars are to form a universe.
Specifying the topology of a galaxy is identical to specifying the
topology of a universe, with one exception:  since a galaxy will be 
connected to other stars or galaxies, you must specify what the inputs
and outputs of a galaxy are.  In fact, a universe is just a galaxy with
no outputs or inputs. 
.pp 
Armed with this information, let us create a simple galaxy: a one-pole
iir filter.  To specify the topology for this galaxy, we will need stars
that add, multiply, and delay samples.  Also, since in BLOSIM connections
can only be made between two stars or galaxies, we will need a star
that implements the fork operation; that is, a star that takes data samples 
from one star and sends copies of them to two or more other stars.
By using the \fBdisplay l\fR command to look at the library, we see
that the stars \fBadd\fR, \fBdelay\fR, \fBmultiply\fR, and \fBfork\fR
will fit the bill.
.pp
First, let's clear out the universe so we
can start from scratch:
.(l
Blosim> \fBnew\fR
.)l
The universe is now empty.  Be very careful when you use this command,
as there is no turning back once you have destroyed the universe.
.pp
Now specify what stars you will need, by issuing the appropriate commands:
.(l
Blosim> \fBstar add0 add\fR
\ Current Block: star add0 add (lib)
Blosim> \fBparam int 1\fR
\ Current Block: star add0 add (lib)
Blosim> \fBstar delay0 delay\fR
\ Current Block: star delay0 delay (lib)
Blosim> \fBstar fork0 fork\fR
\ Current Block: star fork0 fork (lib)
Blosim> \fBparam arg 0\fR
\ Current Block: star fork0 fork (lib)
Blosim> \fBstar multiply0 multiply\fR
\ Current Block: star multiply0 multiply
.)l
The \fBparam\fR specification for the \fBmultiply0\fR star should be noted. 
Rather than specifying a floating-point value,
an \fIarg\fR parameter has been specified.
As we mentioned before, this means that the
actual value to be used for the coefficient
will be taken from the
parameters to the galaxy you are now creating. 
When a copy of this  galaxy is created
for a simulation, it must be given one parameter,
of type floating-point, which will specify the value of
the filter's pole. The \fB0\fR means parameter 0 \*- that
is, the first parameter.
In this way, you can use a galaxy many times, perhaps
to simulate a cascade of first-order filters, with each copy
of the galaxy using a different coefficient value.
.pp
Now make the connections:
.(l
Blosim> \fBconnect input 0 add0 0\fR
\ Current Block: star multiply0 multiply
.)l
Samples that are directed to the 0 input of this galaxy will be 
directed to the 0 input of \fBadd0\fR.
.(l
Blosim> \fBconnect add0 0 fork0 0\fR
\ Current Block: star multiply0 multiply
Blosim> \fBconnect fork0 0 multiply0 0\fR
\ Current Block: star multiply0 multiply
Blosim> \fBconnect multiply0 0 delay0 0\fR
\ Current Block: star multiply0 multiply
Blosim> \fBconnect delay0 0 add0 1\fR
\ Current Block: star multiply0 multiply
Blosim> \fBconnect fork0 1 output 0\fR
\ Current Block: star multiply0 multiply
.)l
Output samples for this galaxy will be taken from output 1 of \fBfork0\fR.
.pp
The specification for a one-pole filter, 
with one input and one output, is now complete. You should now store this
topology, so that we can use it later.  Be sure to specify a 
file name, otherwise BLOSIM will store this galaxy in
the file \*(lquniverse.t\*(rq,
overwriting what we put in there before.
.(l
Blosim> \fBstore filter.t\fR
\ Current Block: star multiply0 multiply
.)l
Note that BLOSIM requires all topology file names to end in \*(lq.t\*(rq.
.pp
It may be worthwhile at this point to show what happens when we try
to run a simulation on a universe with inputs or outputs.
.(l
Blosim> \fBrun\fR
\ run: universe contains input/output connections
\ Current Block: star multiply0 multiply
.)l
.uh "Using Galaxies"
.pp
You are now ready to use the galaxy you have created in a
simulation. In the case of the filter, you can obtain
its impulse response by connecting its input to an \fBimpulse\fR star
and its output to a \fBprintfile\fR star, and running the result.
First, clear the universe again:
.(l
Blosim> \fBnew\fR
.)l
Now, load the universe that was created previously,
containing \fBimpulse0\fR
and \fBprintfile0\fR:
.(l
Blosim> \fBload\fR
\ Current Block: star printfile0 printfile (lib)
.)l
No file name is needed because the topology resides in \(lquniverse.t\(rq.
.pp
The universe now contains the two stars you need (\fBimpulse0\fR and
\fBprintfile0\fR).  Of course, their connections are incorrect, but
you can take care of that in a minute. First, let's set up the 
parameters you'll need. 
A good test value for a pole is 0.90:
.(l
Blosim> \fBparam float 0.90\fR
\ Current Block: star printfile0 printfile (lib)
.)l
Now, use the \fBgalaxy\fR command to create the galaxy:
.(l
Blosim> \fBgalaxy filter0 filter.t\fR
\ Current Block: galaxy filter0 filter.t
.)l
This command is similar to the star command, except that now the
\fBfilter.t\fR argument represents the file where the galaxy's 
topology can be found, rather than the name of a library star.
You can call this galaxy anything you'd like, subject to the same
rules that exist for stars (i.e., no other
star or galaxy in this universe can have the same name);
however, the file name you specify must end in \*(lq.t\*(rq.
.pp
Before you can connect up the universe, you need to remove the 
connection that exists between \fBimpulse0\fR and \fBprintfile\fR
(if you don't believe that this connections exists, use the 
\fBdisplay a\fR to see what BLOSIM thinks).  To remove it, use
the \fBdisconnect\fR command, whose syntax is identical to the
\fBconnect\fR command:
.(l
Blosim> \fBdisconnect impulse0 0 printfile0 0\fR
\ Current Block: galaxy filter0 filter.t
.)l
Now proceed with the necessary connections, just as you did before:
.(l
Blosim> \fBconnect impulse0 0 filter0 0\fR
\ Current Block: galaxy filter0 filter.t
Blosim> \fBconnect filter0 0 printfile0 0\fR
\ Current Block: galaxy filter0 filter.t
.)l
It would probably be a good idea at this point to do a \fBdisplay a\fR
to make sure everything looks good:
.(b
Blosim> \fBdisplay a\fR
\ param float 0.900000
\ galaxy filter0 filter.t

\ param default
\ star printfile0 printfile (lib)

\ param int 20
\ star impulse0 impulse (lib)

\ connect filter0 0 printfile0 0
\ connect impulse0 0 filter0 0
\ Current Block: star printfile0 printfile (lib)
.)b
.pp
Everything looks like it's ready.  Use the \fBrun\fR command to run
the simulation:
.(l
Blosim> \fBrun\fR
.)l
If you did everything correctly up until now, BLOSIM should print twenty
samples of the impulse response of the filter.  You
should now store this filter so that you can use it
in future simulations (put it in \*(lquniverse.t\*(rq):
.(l
Blosim> \fBstore\fR
\ Current Block: star printfile0 printfile (lib)
.)l
.uh "Programming Your Own Stars"
.pp
Of course, BLOSIM would be rather inflexible if you could only build
simulations out of stars in the standard library.  Even our simple
example above is not very useful in a real environment, because
a large filter structure made up completely of adders, multipliers,
and delays would run intolerably slow.
.pp
Fortunately, BLOSIM allows you to design your own stars and use them
in the same way you use stars in the standard library. For this purpose,
a preprocessor to the C language, known as STARGAZE, is provided to
assist in star development. STARGAZE automatically
generates the interface programming that each star 
needs to talk to BLOSIM.  In addition, STARGAZE provides you with
variety of macros that greatly simplify the specification
of the star's function. A complete description of STARGAZE's 
capabilities is discussed in [1]; in this tutorial we
will only touch on a few of its features.
.pp
Following our previous examples,
we will now design a star that has the same function
as the \fBfilter.t\fR galaxy above. To do this, we must first exit
BLOSIM with the quit command:
.(l
Blosim> \fBquit\fR
.)l
Now go into \fIvi\fR, or whatever your favorite editor is, and
get ready to edit the file \*(lqfilter.s\*(rq (all STARGAZE source
files must end in \*(lq.s\*(rq).
.pp
To understand how to write star programs, you must first understand a 
bit about how a star receives and passes on data samples.  From
the star's point of view,  every connection from the star represents
an output buffer full of data samples, while every input connection
represents an input buffer.  During the execution of a simulation,
each star continuously takes data samples one at a time out of
its input buffers, processes them, and puts the results, again
one at a time, onto the output buffers. A simulation stops when
there are no more data samples on any star's input buffers.
Naturally, a star may have no input buffers, in which case it 
just produces data samples (like \fBimpulse0\fR),
or it may have no output buffers, in
which case it just consumes data samples (like \fBprintfile0\fR).
The number of input buffers and output buffers is fixed in the
star's program.  STARGAZE generates code that causes BLOSIM to
print an error message if the number of connections 
made to a star doesn't match the number of input/output buffers.
This error is the \fIuser error code\fR we mentioned above; the
exact message appears after the \fBrun\fR command:
.(l
\ run: star block impulse0 returned non-zero error code (203)
.)l
The offending star is identified, and the exact error code 
is printed.
.pp
To tell STARGAZE that you want your star to have input and/or output
connections, you must declare the corresponding buffers, 
by putting the following lines in your program:
.(b
\fBinput_buffers

     float x;

end

output_buffers

     float y;

end\fR
.)b
Here you have said that the star will have one input buffer
named \fBx\fR
and one output buffer named \fBy\fR.  Both buffers will hold 
floating-point data samples. In our program, \fBx(0)\fR will refer to 
the most recently arrived input sample; \fBx(1)\fR will refer to the
sample that arrived before the most recent; and so forth.
Similarly, \fBy(0)\fR will refer to the next sample
to be passed on; \fBy(1)\fR to the sample
that was just passed on; and so on. To consume a sample on an input
buffer, we use the \fBit_in(\fIn\fB)\fR function: this moves each
input sample on input buffer \fIn\fR up by one, and
returns the number that are left
in that input buffer (Buffers are numbered from \fI0\fR on). 
The \fBit_out(\fIn\fB)\fR function works
in a similar fashion on the output buffer. 
.pp
Our filter star also needs one floating-point parameter, to 
determine what the multiplier value should be.
Like buffers, the number and types of parameters used by a
star must be declared to STARGAZE.
STARGAZE again generates code
that checks for a mismatch and forces an error if there is one.
To declare the parameter, add these lines:
.(l
\fBparameters

      float pole;

end\fR
.)l
In contrast to the buffers, you can access the parameter by using
the identifier \fBpole\fR anytime you choose.
.pp
Now we are ready to write the main code, which is rather simple:
.(b
\fBmain_code

    /* while there are input samples */
    while (it_in(0)) {

         /* advance the output buffer */
         it_out(0); 

         /* compute the new output sample */
         y(0) = x(0) + (pole * y(1)); 
    }

end\fR
.)b
You can now store these lines, and leave the editor (in case you weren't
following along,
the complete program for filter.s is reprinted in Appendix A).
.pp
Now you can run STAGAZE to see if there are any syntax errors
in filter.s:
.(l
% \fBstargaze filter.s\fR
.)l
If there are no error messages, continue on with compilation:
.(l
% \fBcc -c filter.c\fR
.)l
Notice the C file created by STARGAZE has the same name as the STARGAZE
source file, with a \*(lq.c\*(rq instead of
the \*(lq.s\*(rq.  If there are any
compilation errors, it means that you have some errors in filter.s
that STARGAZE did not catch.  Otherwise, you are ready to use this 
star in your simulation.
.uh "Using Your Own Stars"
.pp
We are now ready to try out our new star.  First get back into 
BLOSIM:
.(l
% \fBblosim\fR
\ Blosim 3.0 Topology file: universe.t
Blosim>
.)l
Now load the universe that used the \fBfilter.t\fR 
galaxy:
.(l
Blosim> \fBload\fR
\ Current Block: star printfile0 printfile (lib)
.)l
The \fBremove\fR command removes the Current Block and
all connections from or to it.
Use this command to remove the \fBfilter0\fR galaxy:
.(l
Blosim> \fBforward\fR
\ Current Block: galaxy filter0 filter.t
Blosim> \fBremove\fR
\ Current Block: star printfile0 printfile (lib)
.)l
Next set up the Parameter Stack:
.(l
Blosim> \fBparam float 0.90\fR
\ Current Block: star printfile0 printfile (lib)
.)l
Now use the \fBstar\fR command to create a new \fBfilter0\fR
star, based on the simulation program we just wrote.
In this case, since the star is not in the standard library,
you must specify the name of the file where the compiled
object code for the star can be found.  Since this object code
must now be loaded into BLOSIM's memory, this command will
take a few seconds to execute:
.(l
Blosim> \fBstar filter0 filter.o\fR
\ Current Block: star filter0 filter.o
.)l
Now connect up \fBfilter0\fR:
.(l
Blosim> \fBconnect impulse0 0 filter0 0\fR
\ Current Block: star filter0 filter.o
Blosim> \fBconnect filter0 0 printfile0 0\fR
\ Current Block: star filter0 filter.o
.)l
Now try to run the new star:
.(l
Blosim> \fBrun\fR
.)l
You should now see the same impulse response sequence you saw before.
If you get an error, recheck your connections and your parameter
values.
.uh "Additional Hints"
.pp
By now, you have all the information you need to construct any simulation
you desire. A complete list of BLOSIM's commands is given in Appendix
B; it includes a few commands that we have so far left out.
In particular, you may find the \fBalias\fR and \fBunalias\fR commands
useful. These allow you to redefine BLOSIM's commands, so that you
can save on keystrokes.
.pp
When BLOSIM stores a topology in a file, it stores it as the sequence
of commands necessary to create the specified universe. Thus, the
topology file can be printed or edited like any other text file. You 
can also create universes from the editor, and use BLOSIM only
to run the simulations.  To do
this, invoke BLOSIM with a valid topology file specified:
.(l
% \fBblosim myuniverse.t\fR
.)l
When invoked in this way, BLOSIM builds the universe according to
\fBmyuniverse.t\fR, automatically executes a \fBrun\fR command,
and then exits. The -l option,
.(l
% \fBblosim -l myuniverse.t\fR
.)l
causes BLOSIM to initially load \fBmyuniverse.t\fR, and then prompt
you for commands.
.pp
When BLOSIM starts up, it looks 
for a file named \fB.blosimrc\fR,
first in your current directory,
and then in your home directory.
If this file is found, commands are read from it
and executed before BLOSIM prints the prompt.
You may want to use this file to set up search paths for stars and galaxies
with the \fBpath\fR command, to define aliases, or to 
turn off the Current Block display.  You can also \fBdefine\fR and (thus
pre-load) stars that are not part of the standard library.
.pp
More information about BLOSIM can be found in [2]. 
BLOSIMTOOL, an advanced version of BLOSIM with
graphic display of topologies, is available for
SUN Microsystem workstations.  BLOSIMTOOL is described in [3].
.uh "References"
.sp 2
.in 5
.ti 0
[1] D. G. Messerschmitt, \fISTARGAZE User's Manual\fR (in print)
.br
.sp
.ti 0
[2] D. J. Hait and D. G. Messerschmitt, \fIBLOSIM 3.0 Reference
Manual\fR
.br
.sp
.ti 0
[3] D. J. Hait and D. G. Messerschmitt, \fIA BLOSIMTOOL \*- A Graphic
Interface to BLOSIM\fR
.bp
.uh "Appendix A"
.sp
.(c
\fBFilter.s Listing\fR
.)c
.sp
.(b
input_buffers

     float x;

end

output_buffers

     float y;

end

parameters

     float pole;

end

main_code

    /* while there are input samples */
    while (it_in(0)) {

         /* advance the output buffer */
         it_out(0); 

         /* compute the new output sample */
         y(0) = x(0) + (pole * y(1)); 
    }

end
.)b
.bp
.uh "Appendix B"
.sp
.(c
\fBBLOSIM Command Summary\fR
.)c
.ps 12
.vs 14p
.in 1i
.ti 0
\fBalias\fR
.br
display all aliases
.ti 0
\fBalias \fIalias string\fR
.br
alias the word \fIalias\fR to \fIstring\fR
.ti 0
\fBback\fR
.br
move the Current Block back one
.ti 0
\fBchange p\fR
.br
change the parameters of the Current Block
.ti 0
\fBconnect\fI block1 num1 block2 num2\fR
.br
connect output number \fInum1\fR of \fIblock1\fR
to input number \fInum2\fR of \fIblock2\fR
.ti 0
\fBconnect\fI block1 num1 \fBoutput\fI num2\fR
.br
connect output number \fInum1\fR of \fIblock1\fR 
to output number \fInum2\fR of the parent galaxy
.ti 0
\fBconnect input \fInum1 block2 num2\fR
.br
connect input number \fInum1\fR of
the parent galaxy to input number
\fInum2\fR of \fIblock2\fR
.ti 0
\fBdefine s \fIstarfile\fR
.br
load the star programming in
\fIstarfile\fR into memory
.ti 0
\fBdisconnect\fI block1 num1 block2 num2\fR
.br
disconnect output number \fInum1\fR of
\fIblock1\fR from input number
\fInum2\fR of \fIblock2\fR
.ti 0
\fBdisconnect\fI block1 num1 \fBoutput\fI num2\fR
.br
disconnect output number \fInum1\fR of
\fIblock1\fR from input number 
\fInum2\fR of the parent galaxy
.ti 0
\fBdisconnect input \fInum1 block2 num2\fR
.br 0
disconnect input number \fInum1\fR of
the parent galaxy from input number
\fInum2\fR of \fIblock2\fR
.ti 0
\fBdisplay a\fR
.br
display the entire universe
.ti 0
\fBdisplay l\fR
.br 
display stars in the library
.ti 0
\fBdisplay off\fR
.br
don't display the Current Block
after each command
.ti 0
\fBdisplay on\fR
.br
display the Current Block
after each command
.ti 0
\fBdisplay p\fR
.br
display the Parameter Stack
.ti 0
\fBdisplay s\fR
.br
display the current star list
.ti 0
\fBdisplay\fR
.br
display the Current Block
.ti 0
\fBdown\fR
.br
move down the hierarchy
.ti 0
\fBexit\fR
.br
leave BLOSIM
.ti 0
\fBforward\fR
.br
move the Current Block forward one
.ti 0
\fBgalaxy \fIname topfile\fR
.br
create a galaxy named \fIname\fR with
topology file \fItopfile\fR
.ti 0
\fBinfo\fR
.br
display information about
the Current Block
.ti 0
\fBload\fR
.br
load a topology from the parent
galaxy's topology file
.ti 0
\fBload \fIfile\fR
.br
load a topology from the specified
topology file
.ti 0
\fBnew\fR
.br
destroy the universe
.ti 0
\fBparam\fI type value\fR
.br
place a parameter value on 
the Parameter Stack
.ti 0
\fBpath s \fIpathname\fR
.br
add a directory to the star
path search list
.ti 0
\fBpath g \fIpathname\fR
.br
add a directory to the galaxy
path search list
.ti 0
\fBquit\fR
.br
leave BLOSIM
.ti 0
\fBremove\fR
.br
remove the Current Block
.ti 0
\fBrun\fR
.br
run a simulation
.ti 0
\fBsh \fIcommand\fR
.br
send a command to the Shell
.ti 0
\fBstar \fIname module\fR
.br
create a library star from the specified
library object module
.ti 0
\fBstar \fIname file\fR
.br
create a star from the specified file
.ti 0
\fBstore\fR
.br
store a topology
.ti 0
\fBto\fR \fIname\fR
.br
make \fIname\fR the Current Block
.ti 0
\fBunalias\fI alias\fR
.br
remove an alias
.ti 0
\fBunparam\fR
.br
remove a parameter from 
the Parameter Stack
.ti 0
\fBup\fR
.br
move up
